import shutil

import nipype.interfaces.utility as niu
import nipype.pipeline.engine as pe

import nipype.interfaces.fsl as fsl
import nipype.interfaces.afni as afni
import nipype.interfaces.ants as ants

from nipype.interfaces.niftyreg import reg, regutils
from ..utils.misc import get_elem
from ..utils.utils_nodes import NodeParams, parse_key

from ..nodes.register import (interative_flirt, NMTSubjectAlign,
                              NMTSubjectAlign2, NwarpApplyPriors,
                              animal_warper)

from ..nodes.surface import keep_gcc

from ..nodes.prepare import apply_li_thresh


def create_iterative_register_pipe(
        template_file, template_brain_file, template_mask_file, gm_prob_file,
        wm_prob_file, csf_prob_file, n_iter, name="register_pipe"):
    """
    Registration of template (NMT or other) according to Regis:

    - The iterative FLIRT is between NMT_SS and subject's anat after a quick
    skull-stripping but the anat is more and more refined to corresponds to the
    brain

    - there is also a FNIRT done once, for comparison of the quality of the
    template on the subject's brain

    Not used anymore: corresponds to the IterREGBET provided by Regis in bash
    and wrapped node IterREGBET in nodes/register.py

    #TODO: test if gives the same results as IterREGBET
    """
    register_pipe = pe.Workflow(name=name)

    # creating inputnode
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['anat_file_BET', 'anat_file']),
        name='inputnode')

    # register node
    register = pe.Node(
        niu.Function(input_names=["anat_file", "anat_file_BET",
                                  "template_brain_file",
                                  "template_mask_file", 'n_iter'],
                     output_names=["anat_file_brain", "template_to_anat_file"],
                     function=interative_flirt),
        name="register")

    register.inputs.template_brain_file = template_brain_file
    register.inputs.template_mask_file = template_mask_file
    register.inputs.n_iter = n_iter

    register_pipe.connect(inputnode, 'anat_file', register, 'anat_file')
    register_pipe.connect(inputnode, 'anat_file_BET',
                          register, "anat_file_BET")

    # apply transfo over the 3 tissues:
    # gm
    register_gm = pe.Node(fsl.ApplyXFM(), name="register_gm")

    register_gm.inputs.in_file = gm_prob_file
    register_gm.inputs.apply_xfm = True
    register_gm.inputs.interp = "nearestneighbour"
    register_gm.inputs.output_type = "NIFTI"  # for SPM segment

    register_pipe.connect(register, 'anat_file_brain',
                          register_gm, 'reference')
    register_pipe.connect(register, 'template_to_anat_file',
                          register_gm, "in_matrix_file")

    # wm
    register_wm = pe.Node(fsl.ApplyXFM(), name="register_wm")

    register_wm.inputs.in_file = wm_prob_file
    register_wm.inputs.apply_xfm = True
    register_wm.inputs.interp = "nearestneighbour"
    register_wm.inputs.output_type = "NIFTI"  # for SPM segment

    register_pipe.connect(register, 'anat_file_brain',
                          register_wm, 'reference')
    register_pipe.connect(register, 'template_to_anat_file',
                          register_wm, "in_matrix_file")

    # csf
    register_csf = pe.Node(fsl.ApplyXFM(), name="register_csf")

    register_csf.inputs.in_file = csf_prob_file
    register_csf.inputs.apply_xfm = True
    register_csf.inputs.interp = "nearestneighbour"
    register_csf.inputs.output_type = "NIFTI"  # for SPM segment

    register_pipe.connect(register, 'anat_file_brain',
                          register_csf, 'reference')
    register_pipe.connect(register, 'template_to_anat_file',
                          register_csf, "in_matrix_file")

    def return_list(file1, file2, file3):
        return [file1, file2, file3]

    # merge 3 outputs to a list (...)
    merge_3_files = pe.Node(
        niu.Function(input_names=["file1", "file2", "file3"],
                     output_names=["list3files"],
                     function=return_list),
        name="merge_3_files")

    register_pipe.connect(register_gm, 'out_file', merge_3_files, "file1")
    register_pipe.connect(register_wm, 'out_file', merge_3_files, "file2")
    register_pipe.connect(register_csf, 'out_file', merge_3_files, "file3")

    # same with non linear
    # non linear register between anat and head
    # (FNIRT work directly on head?
    # I thought FLIRT was only skull-stripped brain, is it different? )

    nl_register = pe.Node(fsl.FNIRT(), name="nl_register")
    nl_register.inputs.in_file = template_file

    register_pipe.connect(inputnode, 'anat_file', nl_register, 'ref_file')
    register_pipe.connect(register, 'template_to_anat_file',
                          nl_register, 'affine_file')

    # apply non linear warp to NMT_SS
    nl_apply = pe.Node(fsl.ApplyWarp(), name="nl_apply")
    nl_apply.inputs.in_file = template_brain_file
    register_pipe.connect(inputnode, 'anat_file', nl_apply, 'ref_file')
    register_pipe.connect(nl_register, 'fieldcoeff_file',
                          nl_apply, 'field_file')  # iout from fnirt

    return register_pipe


###############################################################################
def create_register_NMT_pipe(params_template, params={},
                             name="register_NMT_pipe"):
    """Description: Register template to anat with the script NMT_subject_align

    Processing steps:

    - Bias correction (norm_intensity)
    - Deoblique (Refit with deoblique option)
    - NMT_subject_align (see :class:`NMTSubjectAlign \
    <macapype.nodes.register.NMTSubjectAlign>` and :class:`NMTSubjectAlign2 \
    <macapype.nodes.register.NMTSubjectAlign2>` for explanations)
    - apply it to tissues list_priors (NwarpApplyPriors and Allineate)

    Params:
        - norm_intensity (see `N4BiasFieldCorrection <https://\
        nipype.readthedocs.io/en/0.12.1/interfaces/generated/nipype.interfaces\
        .ants.segmentation.html#n4biasfieldcorrection>`_ for arguments)) - \
        also available as :ref:`indiv_params <indiv_params>`

    Inputs:

        inputnode:

            T1:
                T1 file name

            indiv_params:
                dict with individuals parameters for some nodes

        arguments:

            params_template:
                dictionary of info about template

            params:
                dictionary of node sub-parameters (from a json file)

            name:
                pipeline name (default = "register_NMT_pipe")

    Outputs:

        norm_intensity.output_image:
            filled mask after erode

        align_seg_csf.out_file:
            csf template tissue in subject space

        align_seg_gm.out_file:
            grey matter template tissue in subject space

        align_seg_wm.out_file:
            white matter template tissue in subject space
    """

    register_NMT_pipe = pe.Workflow(name=name)

    # creating inputnode
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['T1', 'indiv_params']),
        name='inputnode')

    if "norm_intensity" in params.keys():

        # N4 intensity normalization over brain
        norm_intensity = NodeParams(ants.N4BiasFieldCorrection(),
                                    params=parse_key(params, "norm_intensity"),
                                    name='norm_intensity')

        register_NMT_pipe.connect(inputnode, 'T1',
                                  norm_intensity, "input_image")

        register_NMT_pipe.connect(
            inputnode, ('indiv_params', parse_key, "norm_intensity"),
            norm_intensity, "indiv_params")

    # deoblique (seems mandatory from NMTSubjectAlign)
    if "deoblique" in params.keys():

        deoblique = pe.Node(afni.Refit(deoblique=True), name="deoblique")

        if "norm_intensity" in params.keys():

            register_NMT_pipe.connect(norm_intensity, 'output_image',
                                      deoblique, "in_file")

        else:

            register_NMT_pipe.connect(inputnode, 'T1',
                                      deoblique, "in_file")

    print("which @animal_warper: ", shutil.which("@animal_warper"))

    if "NMT_subject_align" in params.keys():

        if "NMT_version" in params["NMT_subject_align"].keys():
            NMT_version = params["NMT_subject_align"]['NMT_version']
        else:
            print("Warning, for NMT_subject_align, NMT_version is required")
            NMT_version = "v1.3"

        print("*** NMT_version: {}".format(NMT_version))

        print("running NMT_subject_align with version {}".format(NMT_version))

        if NMT_version == "v1.2":
            # align subj to nmt
            NMT_subject_align = NodeParams(
                NMTSubjectAlign(),
                params=parse_key(params, "NMT_subject_align"),
                name='NMT_subject_align')

        elif NMT_version == "v1.3" or NMT_version == "v2.0":
            # align subj to nmt
            NMT_subject_align = NodeParams(
                NMTSubjectAlign2(),
                params=parse_key(params, "NMT_subject_align"),
                name='NMT_subject_align')

        else:
            print("NMT_version {} is not implemented, breaking".format(
                NMT_version))

            exit()

    elif shutil.which("@animal_warper") is not None:
        # TODO AnimalWarper()

        print("running @animal_warper with version {}".format(
            shutil.which("@animal_warper")))

        NMT_subject_align = pe.Node(
            niu.Function(input_names=["T1_file", "NMT_SS_file"],
                         output_names=["aff_file", "warp_file",
                                       "warpinv_file", "transfo_file",
                                       "inv_transfo_file"],
                         function=animal_warper),
            name='NMT_subject_align')
    else:
        print("running default NMT_subject_align version (NMTSubjectAlign2)")

        NMT_subject_align = NodeParams(
            NMTSubjectAlign2(),
            params=parse_key(params, "NMT_subject_align"),
            name='NMT_subject_align')

    NMT_subject_align.inputs.NMT_SS_file = params_template["template_brain"]

    if "deoblique" in params.keys():
        register_NMT_pipe.connect(deoblique, 'out_file',
                                  NMT_subject_align, "T1_file")
    else:

        if "norm_intensity" in params.keys():
            register_NMT_pipe.connect(norm_intensity, 'output_image',
                                      NMT_subject_align, "T1_file")
        else:

            register_NMT_pipe.connect(inputnode, 'T1',
                                      NMT_subject_align, "T1_file")

    # defining list priors
    list_priors = [params_template["template_head"]]

    if "template_seg" in params_template.keys():
        # align_masks
        list_priors.append(params_template["template_seg"])

    else:
        if "template_gm" in params_template.keys():
            list_priors.append(params_template["template_gm"])

        if "template_wm" in params_template.keys():
            list_priors.append(params_template["template_wm"])

        if "template_csf" in params_template.keys():
            list_priors.append(params_template["template_csf"])

    # align_masks
    # "overwrap" of NwarpApply, with specifying the outputs as wished
    align_masks = pe.Node(NwarpApplyPriors(), name='align_masks')
    align_masks.inputs.in_file = list_priors
    align_masks.inputs.out_file = list_priors
    align_masks.inputs.interp = "NN"
    align_masks.inputs.args = "-overwrite"

    register_NMT_pipe.connect(NMT_subject_align, 'aff_file',
                              align_masks, 'master')
    register_NMT_pipe.connect(NMT_subject_align, 'warpinv_file',
                              align_masks, "warp")
    register_NMT_pipe.connect(NMT_subject_align, 'inv_transfo_file',
                              align_masks, "wtransfo")

    outputnode = pe.Node(
        niu.IdentityInterface(fields=['native_template_head',
                                      'native_template_seg',
                                      'native_template_gm',
                                      'native_template_wm',
                                      'native_template_csf']),
        name='outputnode')

    register_NMT_pipe.connect(align_masks, ('out_file', get_elem, 0),
                              outputnode, "native_template_head")
    if "template_seg" in params_template.keys():

        register_NMT_pipe.connect(align_masks, ('out_file', get_elem, 1),
                                  outputnode, "native_template_seg")
    else:
        if "template_csf" in params_template.keys():
            register_NMT_pipe.connect(align_masks,
                                      ('out_file', get_elem, 1),
                                      outputnode, "native_template_gm")

        if "template_gm" in params_template.keys():
            register_NMT_pipe.connect(align_masks,
                                      ('out_file', get_elem, 2),
                                      outputnode, "native_template_wm")

        if "template_wm" in params_template.keys():

            register_NMT_pipe.connect(align_masks,
                                      ('out_file', get_elem, 3),
                                      outputnode, "native_template_csf")
    return register_NMT_pipe


def create_reg_seg_pipe(name="reg_seg_pipe"):

    reg_pipe = pe.Workflow(name=name)

    # creating inputnode
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['native_seg',
                                      'native_threshold_gm',
                                      'native_threshold_wm',
                                      'native_threshold_csf',
                                      'native_prob_gm',
                                      'native_prob_wm',
                                      'native_prob_csf',
                                      'transfo_file',
                                      'ref_image']),
        name='inputnode')

    # align_seg
    align_seg = pe.Node(
        afni.Allineate(), name="align_seg", iterfield=['in_file'])
    align_seg.inputs.final_interpolation = "nearestneighbour"
    align_seg.inputs.overwrite = True
    align_seg.inputs.outputtype = "NIFTI_GZ"

    reg_pipe.connect(inputnode, 'native_seg',
                     align_seg, "in_file")
    reg_pipe.connect(inputnode, 'ref_image', align_seg, "reference")
    reg_pipe.connect(inputnode, 'transfo_file', align_seg, "in_matrix")

    # align_threshold_gm
    align_threshold_gm = pe.Node(
        afni.Allineate(), name="align_threshold_gm", iterfield=['in_file'])
    align_threshold_gm.inputs.final_interpolation = "nearestneighbour"
    align_threshold_gm.inputs.overwrite = True
    align_threshold_gm.inputs.outputtype = "NIFTI_GZ"

    reg_pipe.connect(inputnode, 'native_threshold_gm',
                     align_threshold_gm, "in_file")
    reg_pipe.connect(inputnode, 'ref_image', align_threshold_gm, "reference")
    reg_pipe.connect(inputnode, 'transfo_file',
                     align_threshold_gm, "in_matrix")

    # align_threshold_wm
    align_threshold_wm = pe.Node(
        afni.Allineate(), name="align_threshold_wm", iterfield=['in_file'])
    align_threshold_wm.inputs.final_interpolation = "nearestneighbour"
    align_threshold_wm.inputs.overwrite = True
    align_threshold_wm.inputs.outputtype = "NIFTI_GZ"

    reg_pipe.connect(inputnode, 'native_threshold_wm',
                     align_threshold_wm, "in_file")
    reg_pipe.connect(inputnode, 'ref_image', align_threshold_wm, "reference")
    reg_pipe.connect(inputnode, 'transfo_file',
                     align_threshold_wm, "in_matrix")

    # align_threshold_csf
    align_threshold_csf = pe.Node(
        afni.Allineate(), name="align_threshold_csf", iterfield=['in_file'])
    align_threshold_csf.inputs.final_interpolation = "nearestneighbour"
    align_threshold_csf.inputs.overwrite = True
    align_threshold_csf.inputs.outputtype = "NIFTI_GZ"

    reg_pipe.connect(inputnode, 'native_threshold_csf',
                     align_threshold_csf, "in_file")
    reg_pipe.connect(inputnode, 'ref_image', align_threshold_csf, "reference")
    reg_pipe.connect(inputnode, 'transfo_file',
                     align_threshold_csf, "in_matrix")

    # align_prob_gm
    align_prob_gm = pe.Node(
        afni.Allineate(), name="align_prob_gm", iterfield=['in_file'])
    align_prob_gm.inputs.final_interpolation = "nearestneighbour"
    align_prob_gm.inputs.overwrite = True
    align_prob_gm.inputs.outputtype = "NIFTI_GZ"

    reg_pipe.connect(inputnode, 'native_prob_gm',
                     align_prob_gm, "in_file")
    reg_pipe.connect(inputnode, 'ref_image', align_prob_gm, "reference")
    reg_pipe.connect(inputnode, 'transfo_file', align_prob_gm, "in_matrix")

    # align_prob_wm
    align_prob_wm = pe.Node(
        afni.Allineate(), name="align_prob_wm", iterfield=['in_file'])
    align_prob_wm.inputs.final_interpolation = "nearestneighbour"
    align_prob_wm.inputs.overwrite = True
    align_prob_wm.inputs.outputtype = "NIFTI_GZ"

    reg_pipe.connect(inputnode, 'native_prob_wm',
                     align_prob_wm, "in_file")
    reg_pipe.connect(inputnode, 'ref_image', align_prob_wm, "reference")
    reg_pipe.connect(inputnode, 'transfo_file', align_prob_wm, "in_matrix")

    # align_prob_csf
    align_prob_csf = pe.Node(
        afni.Allineate(), name="align_prob_csf", iterfield=['in_file'])
    align_prob_csf.inputs.final_interpolation = "nearestneighbour"
    align_prob_csf.inputs.overwrite = True
    align_prob_csf.inputs.outputtype = "NIFTI_GZ"

    reg_pipe.connect(inputnode, 'native_prob_csf',
                     align_prob_csf, "in_file")
    reg_pipe.connect(inputnode, 'ref_image', align_prob_csf, "reference")
    reg_pipe.connect(inputnode, 'transfo_file', align_prob_csf, "in_matrix")

    outputnode = pe.Node(
        niu.IdentityInterface(fields=['norm_seg', 'norm_threshold_gm',
                                      'norm_threshold_wm',
                                      'norm_threshold_csf',
                                      'norm_prob_gm', 'norm_prob_wm',
                                      'norm_prob_csf']),
        name='outputnode')

    reg_pipe.connect(align_seg, 'out_file', outputnode, "norm_seg")

    reg_pipe.connect(align_threshold_gm, 'out_file',
                     outputnode, "norm_threshold_gm")
    reg_pipe.connect(align_threshold_wm, 'out_file',
                     outputnode, "norm_threshold_wm")
    reg_pipe.connect(align_threshold_csf, 'out_file',
                     outputnode, "norm_threshold_csf")

    reg_pipe.connect(align_prob_gm, 'out_file', outputnode, "norm_prob_gm")
    reg_pipe.connect(align_prob_wm, 'out_file', outputnode, "norm_prob_wm")
    reg_pipe.connect(align_prob_csf, 'out_file', outputnode, "norm_prob_csf")

    return reg_pipe


# used for registration
def _create_remove_capsule_pipeline(name="remove_capsule_pipe", params={}):
    """
    By david:
    li_thresholding
    gcc
    """
    # creating pipeline
    remove_caps_pipe = pe.Workflow(name=name)

    # Creating input node
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['orig_img', 'indiv_params']),
        name='inputnode'
    )

    # lithresholding
    li_thresh = pe.Node(
        niu.Function(input_names=['orig_img_file'],
                     output_names=['lithr_img_file'],
                     function=apply_li_thresh),
        name="li_thresholding")

    remove_caps_pipe.connect(inputnode, 'orig_img', li_thresh, 'orig_img_file')

    # gcc

    gcc_mask = pe.Node(
        niu.Function(input_names=['nii_file'],
                     output_names=['gcc_nii_file'],
                     function=keep_gcc),
        name="gcc_mask")

    remove_caps_pipe.connect(li_thresh, 'lithr_img_file',
                             gcc_mask, 'nii_file')

    # masking original_image
    mask_capsule = pe.Node(fsl.ApplyMask(), name='mask_capsule')

    remove_caps_pipe.connect(inputnode, 'orig_img',
                             mask_capsule, 'in_file')
    remove_caps_pipe.connect(gcc_mask, 'gcc_nii_file',
                             mask_capsule, 'mask_file')

    return remove_caps_pipe


def create_crop_aladin_pipe(name="crop_aladin_pipe", params={}):

    reg_pipe = pe.Workflow(name=name)

    # creating inputnode
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['native_T1',
                                      'stereo_template_T1',
                                      'indiv_params']),
        name='inputnode')

    # outputnode
    outputnode = pe.Node(
        niu.IdentityInterface(fields=['stereo_T1',
                                      'native_to_stereo_trans']),
        name='outputnode')

    if "remove_capsule_pipe" in params:
        remove_capsule_pipe = _create_remove_capsule_pipeline(
            params=params["remove_capsule_pipe"])

        reg_pipe.connect(
                inputnode, 'native_T1',
                remove_capsule_pipe, 'inputnode.orig_img')

    # align T1 on template
    reg_T1_on_template = NodeParams(
        reg.RegAladin(),
        params=parse_key(params, "reg_T1_on_template"),
        name='reg_T1_on_template')

    if "remove_capsule_pipe" in params:
        reg_pipe.connect(
            remove_capsule_pipe, 'mask_capsule.out_file',
            reg_T1_on_template, "flo_file")
    else:

        reg_pipe.connect(
            inputnode, 'native_T1',
            reg_T1_on_template, "flo_file")

    reg_pipe.connect(
        inputnode, 'stereo_template_T1',
        # reg_pipe.connect(pad_template_T1, 'img_padded_file',
        reg_T1_on_template, "ref_file")

    reg_pipe.connect(
        inputnode, ('indiv_params', parse_key, "reg_T1_on_template"),
        reg_T1_on_template, "indiv_params")

    # reg_T1_on_template2
    if "reg_T1_on_template2" in params.keys():
        # second align T1 on template (sometimes needed)
        reg_T1_on_template2 = NodeParams(
            reg.RegAladin(),
            params=parse_key(params, "reg_T1_on_template2"),
            name='reg_T1_on_template2')

        reg_pipe.connect(reg_T1_on_template, 'res_file',
                         reg_T1_on_template2, "flo_file")

        reg_pipe.connect(
            # pad_template_T1, 'img_padded_file',
            inputnode, 'stereo_template_T1',
            reg_T1_on_template2, "ref_file")

        reg_pipe.connect(
            inputnode, ('indiv_params', parse_key, "reg_T1_on_template2"),
            reg_T1_on_template2, "indiv_params")

        # compose_transfo
        compose_transfo = pe.Node(regutils.RegTransform(),
                                  name="compose_transfo")

        reg_pipe.connect(reg_T1_on_template, 'aff_file',
                         compose_transfo, "comp_input2")

        reg_pipe.connect(reg_T1_on_template2, 'aff_file',
                         compose_transfo, "comp_input")

    # padding image to have zeros and non nans
    pad_image_T1 = pe.Node(
        ants.utils.ImageMath(),
        name="pad_image_T1")

    pad_image_T1.inputs.copy_header = True
    pad_image_T1.inputs.operation = "PadImage"
    pad_image_T1.inputs.op2 = '200'

    reg_pipe.connect(
        inputnode, 'native_T1',
        pad_image_T1, "op1")

    # resampling using transfo on much bigger image
    reg_resample_T1 = pe.Node(
        regutils.RegResample(pad_val=0.0),
        name="reg_resample_T1")

    # transfo
    if "reg_T1_on_template2" in params.keys():
        reg_pipe.connect(
            compose_transfo, 'out_file',
            reg_resample_T1, 'trans_file')

    else:
        reg_pipe.connect(
            reg_T1_on_template, 'aff_file',
            reg_resample_T1, 'trans_file')

    reg_pipe.connect(
        pad_image_T1, 'output_image',
        reg_resample_T1, "flo_file")

    reg_pipe.connect(
            inputnode, 'stereo_template_T1',
            reg_resample_T1, "ref_file")

    # outputnode
    reg_pipe.connect(reg_resample_T1, 'out_file',
                     outputnode, "stereo_T1")

    if "reg_T1_on_template2" in params.keys():
        reg_pipe.connect(
            compose_transfo, 'out_file',
            outputnode, "native_to_stereo_trans")

    else:
        reg_pipe.connect(
            reg_T1_on_template, 'aff_file',
            outputnode, "native_to_stereo_trans")

    return reg_pipe




import os.path as op

from macapype.utils.utils_tests import (make_tmp_dir, load_test_data,
                                        format_template)

from macapype.utils.misc import parse_key
from macapype.pipelines.surface import (create_nii_to_mesh_pipe)


data_path = make_tmp_dir()


def test_create_surface_pipe():

    # params
    params = {
        "nii_to_mesh_pipe":
        {
                "split_hemi_pipe":
                {
                },

        }
    }

    # params_template
    template_name = "haiko89_template"
    template_dir = load_test_data(template_name)

    params_template = format_template(template_dir, template_name)

    # running workflow
    segment_pnh = create_nii_to_mesh_pipe(
        params=parse_key(params, "nii_to_mesh_pipe"),
        params_template=params_template,
        name="nii_to_mesh_pipe")

    segment_pnh.base_dir = data_path

    segment_pnh.write_graph(graph2use="colored")
    assert op.exists(op.join(data_path,
                             "nii_to_mesh_pipe",
                             "graph.png"))




import nipype.pipeline.engine as pe
import nipype.interfaces.utility as niu


def rename_all_brain_derivatives(params, main_workflow, segment_pnh_pipe,
                                 datasink, pref_deriv, parse_str,
                                 pad, ssoft, datatypes):

    # default: projection in stereo
    # rename stereo_T1
    rename_stereo_T1 = pe.Node(
        niu.Rename(),
        name="rename_stereo_T1")
    rename_stereo_T1.inputs.format_string = \
        pref_deriv + "_space-stereo_T1w"
    rename_stereo_T1.inputs.parse_string = parse_str
    rename_stereo_T1.inputs.keep_ext = True

    main_workflow.connect(
        segment_pnh_pipe, 'outputnode.stereo_T1',
        rename_stereo_T1, 'in_file')

    main_workflow.connect(
        rename_stereo_T1, 'out_file',
        datasink, '@stereo_T1')

    if 't2' in datatypes:

        # rename stereo_T2
        rename_stereo_T2 = pe.Node(
            niu.Rename(),
            name="rename_stereo_T2")
        rename_stereo_T2.inputs.format_string = \
            pref_deriv + "_space-stereo_T2w"
        rename_stereo_T2.inputs.parse_string = parse_str
        rename_stereo_T2.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_T2',
            rename_stereo_T2, 'in_file')

        main_workflow.connect(
            rename_stereo_T2, 'out_file',
            datasink, '@stereo_T2')

    # transfo to native to template and inverse

    # rename trans
    rename_trans = pe.Node(
        niu.Rename(),
        name="rename_trans")
    rename_trans.inputs.format_string = \
        pref_deriv + "_space-native_target-stereo_affine"
    rename_trans.inputs.parse_string = parse_str
    rename_trans.inputs.keep_ext = True

    main_workflow.connect(
        segment_pnh_pipe, 'outputnode.native_to_stereo_trans',
        rename_trans, 'in_file')

    main_workflow.connect(
        rename_trans, 'out_file',
        datasink, '@native_to_stereo_trans')

    # rename inv_trans
    rename_inv_trans = pe.Node(
        niu.Rename(),
        name="rename_inv_trans")
    rename_inv_trans.inputs.format_string = \
        pref_deriv + "_space-stereo_target-native_affine"
    rename_inv_trans.inputs.parse_string = parse_str
    rename_inv_trans.inputs.keep_ext = True

    main_workflow.connect(
        segment_pnh_pipe, 'outputnode.stereo_to_native_trans',
        rename_inv_trans, 'in_file')

    main_workflow.connect(
        rename_inv_trans, 'out_file',
        datasink, '@stereo_to_native_trans')

    if "denoise" in params["short_preparation_pipe"].keys():

        rename_stereo_denoised_T1 = pe.Node(
            niu.Rename(),
            name="rename_stereo_denoised_T1")
        rename_stereo_denoised_T1.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-denoised_T1w"
        rename_stereo_denoised_T1.inputs.parse_string = parse_str
        rename_stereo_denoised_T1.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_denoised_T1',
            rename_stereo_denoised_T1, 'in_file')

        main_workflow.connect(
            rename_stereo_denoised_T1, 'out_file',
            datasink, '@stereo_denoised_T1')

        if 't2' in datatypes:

            # rename stereo_denoised_T2
            rename_stereo_denoised_T2 = pe.Node(
                niu.Rename(),
                name="rename_stereo_denoised_T2")
            rename_stereo_denoised_T2.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-denoised_T2w"
            rename_stereo_denoised_T2.inputs.parse_string = parse_str
            rename_stereo_denoised_T2.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.stereo_denoised_T2',
                rename_stereo_denoised_T2, 'in_file')

            main_workflow.connect(
                rename_stereo_denoised_T2, 'out_file',
                datasink, '@stereo_denoised_T2')

    if "pad_template" in params["short_preparation_pipe"].keys():

        rename_stereo_padded_T1 = pe.Node(
            niu.Rename(),
            name="rename_stereo_padded_T1")
        rename_stereo_padded_T1.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-pad_T1w"
        rename_stereo_padded_T1.inputs.parse_string = parse_str
        rename_stereo_padded_T1.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_padded_T1',
            rename_stereo_padded_T1, 'in_file')

        main_workflow.connect(
            rename_stereo_padded_T1, 'out_file',
            datasink, '@stereo_padded_T1')

        if 't2' in datatypes:

            # rename stereo_padded_T2
            rename_stereo_padded_T2 = pe.Node(
                niu.Rename(),
                name="rename_stereo_padded_T2")
            rename_stereo_padded_T2.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-pad_T2w"
            rename_stereo_padded_T2.inputs.parse_string = parse_str
            rename_stereo_padded_T2.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.stereo_padded_T2',
                rename_stereo_padded_T2, 'in_file')

            main_workflow.connect(
                rename_stereo_padded_T2, 'out_file',
                datasink, '@stereo_padded_T2')

    if ("fast" in params
            or "N4debias" in params
            or "correct_bias_pipe" in params):

        # rename debiased_T1
        rename_stereo_debiased_T1 = pe.Node(
            niu.Rename(),
            name="rename_stereo_debiased_T1")
        rename_stereo_debiased_T1.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-debiased_T1w"
        rename_stereo_debiased_T1.inputs.parse_string = parse_str
        rename_stereo_debiased_T1.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_debiased_T1',
            rename_stereo_debiased_T1, 'in_file')

        main_workflow.connect(
            rename_stereo_debiased_T1, 'out_file',
            datasink, '@stereo_debiased_T1')

        if 't2' in datatypes:

            # rename debiased_T2
            rename_stereo_debiased_T2 = pe.Node(
                niu.Rename(),
                name="rename_stereo_debiased_T2")
            rename_stereo_debiased_T2.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-debiased_T2w"
            rename_stereo_debiased_T2.inputs.parse_string = parse_str
            rename_stereo_debiased_T2.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.stereo_debiased_T2',
                rename_stereo_debiased_T2, 'in_file')

            main_workflow.connect(
                rename_stereo_debiased_T2, 'out_file',
                datasink, '@stereo_debiased_T2')

    if "extract_pipe" in params.keys():

        # rename brain_mask
        rename_stereo_brain_mask = pe.Node(
            niu.Rename(), name="rename_stereo_brain_mask")
        rename_stereo_brain_mask.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-brain_mask"
        rename_stereo_brain_mask.inputs.parse_string = parse_str
        rename_stereo_brain_mask.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_brain_mask',
            rename_stereo_brain_mask, 'in_file')

        main_workflow.connect(
            rename_stereo_brain_mask, 'out_file',
            datasink, '@stereo_brain_mask')

        if "pad_template" in params["short_preparation_pipe"].keys():

            rename_stereo_padded_brain_mask = pe.Node(
                niu.Rename(),
                name="rename_stereo_padded_brain_mask")
            rename_stereo_padded_brain_mask.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-pad_desc-brain_mask"
            rename_stereo_padded_brain_mask.inputs.parse_string = parse_str
            rename_stereo_padded_brain_mask.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.stereo_padded_brain_mask',
                rename_stereo_padded_brain_mask, 'in_file')

            main_workflow.connect(
                rename_stereo_padded_brain_mask, 'out_file',
                datasink, '@stereo_padded_brain_mask')

    if "masked_correct_bias_pipe" in params.keys():

        # rename masked_debiased_T1
        rename_stereo_masked_debiased_T1 = pe.Node(
            niu.Rename(),
            name="rename_stereo_masked_debiased_T1")
        rename_stereo_masked_debiased_T1.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-debiased_desc-brain_T1w"
        rename_stereo_masked_debiased_T1.inputs.parse_string = parse_str
        rename_stereo_masked_debiased_T1.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_masked_debiased_T1',
            rename_stereo_masked_debiased_T1, 'in_file')

        main_workflow.connect(
            rename_stereo_masked_debiased_T1, 'out_file',
            datasink, '@stereo_masked_debiased_T1')

        if 't2' in datatypes:

            # rename masked_debiased_T2
            rename_stereo_masked_debiased_T2 = pe.Node(
                niu.Rename(),
                name="rename_stereo_masked_debiased_T2")
            rename_stereo_masked_debiased_T2.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-debiased_desc-brain_T2w"
            rename_stereo_masked_debiased_T2.inputs.parse_string = \
                parse_str
            rename_stereo_masked_debiased_T2.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.stereo_masked_debiased_T2',
                rename_stereo_masked_debiased_T2, 'in_file')

            main_workflow.connect(
                rename_stereo_masked_debiased_T2, 'out_file',
                datasink, '@stereo_masked_debiased_T2')

    if "debias" in params.keys():

        # rename masked_debiased_T1
        rename_stereo_masked_debiased_T1 = pe.Node(
            niu.Rename(),
            name="rename_stereo_masked_debiased_T1")
        rename_stereo_masked_debiased_T1.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-debiased_desc-brain_T1w"
        rename_stereo_masked_debiased_T1.inputs.parse_string = parse_str
        rename_stereo_masked_debiased_T1.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_masked_debiased_T1',
            rename_stereo_masked_debiased_T1, 'in_file')

        main_workflow.connect(
            rename_stereo_masked_debiased_T1, 'out_file',
            datasink, '@stereo_masked_debiased_T1')

        if 't2' in datatypes:
            # rename masked_debiased_T2
            rename_stereo_masked_debiased_T2 = pe.Node(
                niu.Rename(),
                name="rename_stereo_masked_debiased_T2")
            rename_stereo_masked_debiased_T2.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-debiased_desc-brain_T2w"
            rename_stereo_masked_debiased_T2.inputs.parse_string = \
                parse_str
            rename_stereo_masked_debiased_T2.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.stereo_masked_debiased_T2',
                rename_stereo_masked_debiased_T2, 'in_file')

            main_workflow.connect(
                rename_stereo_masked_debiased_T2, 'out_file',
                datasink, '@stereo_masked_debiased_T2')

        if 'extract_pipe' not in params.keys():

            # rename brain_mask
            rename_stereo_brain_mask = pe.Node(
                niu.Rename(), name="rename_stereo_brain_mask")
            rename_stereo_brain_mask.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-brain_mask"
            rename_stereo_brain_mask.inputs.parse_string = parse_str
            rename_stereo_brain_mask.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.stereo_brain_mask',
                rename_stereo_brain_mask, 'in_file')

            main_workflow.connect(
                rename_stereo_brain_mask, 'out_file',
                datasink, '@stereo_brain_mask')

    if "brain_segment_pipe" in params or "old_segment_pipe" in params:

        # rename prob_wm
        rename_stereo_prob_wm = pe.Node(niu.Rename(),
                                        name="rename_stereo_prob_wm")
        rename_stereo_prob_wm.inputs.format_string = \
            pref_deriv + "_space-stereo_label-WM_probseg"
        rename_stereo_prob_wm.inputs.parse_string = parse_str
        rename_stereo_prob_wm.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_prob_wm',
            rename_stereo_prob_wm, 'in_file')

        main_workflow.connect(
            rename_stereo_prob_wm, 'out_file',
            datasink, '@stereo_prob_wm')

        # rename prob_gm
        rename_stereo_prob_gm = pe.Node(niu.Rename(),
                                        name="rename_stereo_prob_gm")
        rename_stereo_prob_gm.inputs.format_string = \
            pref_deriv + "_space-stereo_label-GM_probseg"
        rename_stereo_prob_gm.inputs.parse_string = parse_str
        rename_stereo_prob_gm.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_prob_gm',
            rename_stereo_prob_gm, 'in_file')

        main_workflow.connect(
            rename_stereo_prob_gm, 'out_file',
            datasink, '@stereo_prob_gm')

        # rename prob_csf
        rename_stereo_prob_csf = pe.Node(niu.Rename(),
                                         name="rename_stereo_prob_csf")
        rename_stereo_prob_csf.inputs.format_string = \
            pref_deriv + "_space-stereo_label-CSF_probseg"
        rename_stereo_prob_csf.inputs.parse_string = parse_str
        rename_stereo_prob_csf.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_prob_csf',
            rename_stereo_prob_csf, 'in_file')

        main_workflow.connect(
            rename_stereo_prob_csf, 'out_file',
            datasink, '@stereo_prob_csf')

    if "brain_segment_pipe" in params:

        # rename segmented_brain_mask
        rename_stereo_segmented_brain_mask = pe.Node(
            niu.Rename(),
            name="rename_stereo_segmented_brain_mask")
        rename_stereo_segmented_brain_mask.inputs.format_string = \
            pref_deriv + "_space-stereo_desc-brain_dseg"
        rename_stereo_segmented_brain_mask.inputs.parse_string = parse_str
        rename_stereo_segmented_brain_mask.inputs.keep_ext = True

        main_workflow.connect(
            segment_pnh_pipe, 'outputnode.stereo_segmented_brain_mask',
            rename_stereo_segmented_brain_mask, 'in_file')

        main_workflow.connect(
            rename_stereo_segmented_brain_mask, 'out_file',
            datasink, '@stereo_segmented_brain_mask')

        if "pad_template" in params["short_preparation_pipe"].keys():

            rename_stereo_padded_segmented_brain_mask = pe.Node(
                niu.Rename(),
                name="rename_stereo_padded_segmented_brain_mask")
            rename_stereo_padded_segmented_brain_mask.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-pad_desc-brain_dseg"
            rename_stereo_padded_segmented_brain_mask.inputs.parse_string = \
                parse_str
            rename_stereo_padded_segmented_brain_mask.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe,
                'outputnode.stereo_padded_segmented_brain_mask',
                rename_stereo_padded_segmented_brain_mask, 'in_file')

            main_workflow.connect(
                rename_stereo_padded_segmented_brain_mask, 'out_file',
                datasink, '@stereo_padded_segmented_brain_mask')

        # rename 5tt
        if "export_5tt_pipe" in params["brain_segment_pipe"].keys():
            rename_stereo_gen_5tt = pe.Node(
                niu.Rename(),
                name="rename_stereo_gen_5tt")
            rename_stereo_gen_5tt.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-5tt_dseg"
            rename_stereo_gen_5tt.inputs.parse_string = parse_str
            rename_stereo_gen_5tt.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.stereo_gen_5tt',
                rename_stereo_gen_5tt, 'in_file')

            main_workflow.connect(
                rename_stereo_gen_5tt, 'out_file',
                datasink, '@stereo_gen_5tt')

        if "nii2mesh_brain_pipe" in params["brain_segment_pipe"] \
                or "IsoSurface_brain_pipe" in params["brain_segment_pipe"]:

            print("Renaming wmgm_stl file")

            rename_stereo_wmgm_stl = pe.Node(
                niu.Rename(), name="rename_stereo_wmgm_stl")
            rename_stereo_wmgm_stl.inputs.format_string = \
                pref_deriv + "_desc-wmgm_mask"
            rename_stereo_wmgm_stl.inputs.parse_string = parse_str
            rename_stereo_wmgm_stl.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.wmgm_stl',
                rename_stereo_wmgm_stl, 'in_file')

            main_workflow.connect(
                rename_stereo_wmgm_stl, 'out_file',
                datasink, '@stereo_wmgm_stl')

            print("Renaming wmgm_nii file")
            rename_stereo_wmgm_mask = pe.Node(
                niu.Rename(), name="rename_stereo_wmgm_mask")
            rename_stereo_wmgm_mask.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-wmgm_mask"
            rename_stereo_wmgm_mask.inputs.parse_string = parse_str
            rename_stereo_wmgm_mask.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.stereo_wmgm_mask',
                rename_stereo_wmgm_mask, 'in_file')

            main_workflow.connect(
                rename_stereo_wmgm_mask, 'out_file',
                datasink, '@stereo_wmgm_mask')

    elif "old_segment_pipe" in params.keys():

        if "mask_from_seg_pipe" in params.keys():

            # rename segmented_brain_mask
            rename_stereo_segmented_brain_mask = pe.Node(
                niu.Rename(),
                name="rename_stereo_segmented_brain_mask")
            rename_stereo_segmented_brain_mask.inputs.format_string = \
                pref_deriv + "_space-stereo_desc-brain_dseg"
            rename_stereo_segmented_brain_mask.inputs.parse_string = \
                parse_str
            rename_stereo_segmented_brain_mask.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.stereo_segmented_brain_mask',
                rename_stereo_segmented_brain_mask, 'in_file')

            main_workflow.connect(
                rename_stereo_segmented_brain_mask, 'out_file',
                datasink, '@stereo_segmented_brain_mask')

            # rename 5tt
            if "export_5tt_pipe" in params["old_segment_pipe"].keys():
                rename_stereo_gen_5tt = pe.Node(
                    niu.Rename(), name="rename_stereo_gen_5tt")
                rename_stereo_gen_5tt.inputs.format_string = \
                    pref_deriv + "_space-stereo_desc-5tt_dseg"
                rename_stereo_gen_5tt.inputs.parse_string = parse_str
                rename_stereo_gen_5tt.inputs.keep_ext = True

                main_workflow.connect(
                    segment_pnh_pipe, 'outputnode.stereo_gen_5tt',
                    rename_stereo_gen_5tt, 'in_file')

                main_workflow.connect(
                    rename_stereo_gen_5tt, 'out_file',
                    datasink, '@stereo_gen_5tt')

    # if pad back to native is defined
    if pad:

        # after some processing
        if ("fast" in params
                or "N4debias" in params
                or "correct_bias_pipe" in params):

            # rename debiased_T1
            rename_native_debiased_T1 = pe.Node(
                niu.Rename(),
                name="rename_native_debiased_T1")
            rename_native_debiased_T1.inputs.format_string = \
                pref_deriv + "_space-native_desc-debiased_T1w"
            rename_native_debiased_T1.inputs.parse_string = parse_str
            rename_native_debiased_T1.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.native_debiased_T1',
                rename_native_debiased_T1, 'in_file')

            main_workflow.connect(
                rename_native_debiased_T1, 'out_file',
                datasink, '@native_debiased_T1')

            if 't2' in datatypes:

                # rename debiased_T2
                rename_native_debiased_T2 = pe.Node(
                    niu.Rename(),
                    name="rename_native_debiased_T2")
                rename_native_debiased_T2.inputs.format_string = \
                    pref_deriv + "_space-native_desc-debiased_T2w"
                rename_native_debiased_T2.inputs.parse_string = parse_str
                rename_native_debiased_T2.inputs.keep_ext = True

                main_workflow.connect(
                    segment_pnh_pipe, 'outputnode.native_debiased_T2',
                    rename_native_debiased_T2, 'in_file')

                main_workflow.connect(
                    rename_native_debiased_T2, 'out_file',
                    datasink, '@native_debiased_T2')

        if "extract_pipe" in params.keys():

            # rename brain_mask
            rename_native_brain_mask = pe.Node(
                niu.Rename(), name="rename_native_brain_mask")
            rename_native_brain_mask.inputs.format_string = \
                pref_deriv + "_space-native_desc-brain_mask"
            rename_native_brain_mask.inputs.parse_string = parse_str
            rename_native_brain_mask.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.native_brain_mask',
                rename_native_brain_mask, 'in_file')

            main_workflow.connect(
                rename_native_brain_mask, 'out_file',
                datasink, '@native_brain_mask')

        if "masked_correct_bias_pipe" in params.keys():

            # rename masked_debiased_T1
            rename_native_masked_debiased_T1 = pe.Node(
                niu.Rename(),
                name="rename_native_masked_debiased_T1")
            rename_native_masked_debiased_T1.inputs.format_string = \
                pref_deriv + "_space-native_desc-debiased_desc-brain_T1w"
            rename_native_masked_debiased_T1.inputs.parse_string = parse_str
            rename_native_masked_debiased_T1.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.native_masked_debiased_T1',
                rename_native_masked_debiased_T1, 'in_file')

            main_workflow.connect(
                rename_native_masked_debiased_T1, 'out_file',
                datasink, '@native_masked_debiased_T1')

            if 't2' in datatypes:

                # rename masked_debiased_T2
                rename_native_masked_debiased_T2 = pe.Node(
                    niu.Rename(),
                    name="rename_native_masked_debiased_T2")
                rename_native_masked_debiased_T2.inputs.format_string = \
                    pref_deriv + "_space-native_desc-debiased_desc-brain_T2w"
                rename_native_masked_debiased_T2.inputs.parse_string = \
                    parse_str
                rename_native_masked_debiased_T2.inputs.keep_ext = True

                main_workflow.connect(
                    segment_pnh_pipe, 'outputnode.native_masked_debiased_T2',
                    rename_native_masked_debiased_T2, 'in_file')

                main_workflow.connect(
                    rename_native_masked_debiased_T2, 'out_file',
                    datasink, '@native_masked_debiased_T2')

        if "debias" in params.keys():

            # rename masked_debiased_T1
            rename_native_masked_debiased_T1 = pe.Node(
                niu.Rename(),
                name="rename_native_masked_debiased_T1")
            rename_native_masked_debiased_T1.inputs.format_string = \
                pref_deriv + "_space-native_desc-debiased_desc-brain_T1w"
            rename_native_masked_debiased_T1.inputs.parse_string = parse_str
            rename_native_masked_debiased_T1.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.native_masked_debiased_T1',
                rename_native_masked_debiased_T1, 'in_file')

            main_workflow.connect(
                rename_native_masked_debiased_T1, 'out_file',
                datasink, '@native_masked_debiased_T1')

            if 't2' in datatypes:
                # rename masked_debiased_T2
                rename_native_masked_debiased_T2 = pe.Node(
                    niu.Rename(),
                    name="rename_native_masked_debiased_T2")
                rename_native_masked_debiased_T2.inputs.format_string = \
                    pref_deriv + \
                    "_space-native_desc-debiased_desc-brain_T2w"

                rename_native_masked_debiased_T2.inputs.parse_string = \
                    parse_str
                rename_native_masked_debiased_T2.inputs.keep_ext = True

                main_workflow.connect(
                    segment_pnh_pipe, 'outputnode.native_masked_debiased_T2',
                    rename_native_masked_debiased_T2, 'in_file')

                main_workflow.connect(
                    rename_native_masked_debiased_T2, 'out_file',
                    datasink, '@native_masked_debiased_T2')

            if 'extract_pipe' not in params.keys():

                # rename brain_mask
                rename_native_brain_mask = pe.Node(
                    niu.Rename(), name="rename_native_brain_mask")
                rename_native_brain_mask.inputs.format_string = \
                    pref_deriv + "_space-native_desc-brain_mask"
                rename_native_brain_mask.inputs.parse_string = parse_str
                rename_native_brain_mask.inputs.keep_ext = True

                main_workflow.connect(
                    segment_pnh_pipe, 'outputnode.native_brain_mask',
                    rename_native_brain_mask, 'in_file')

                main_workflow.connect(
                    rename_native_brain_mask, 'out_file',
                    datasink, '@native_brain_mask')

        if "brain_segment_pipe" in params or "old_segment_pipe" in params:

            # rename prob_wm
            rename_native_prob_wm = pe.Node(
                niu.Rename(), name="rename_native_prob_wm")
            rename_native_prob_wm.inputs.format_string = \
                pref_deriv + "_space-native_label-WM_probseg"
            rename_native_prob_wm.inputs.parse_string = parse_str
            rename_native_prob_wm.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.native_prob_wm',
                rename_native_prob_wm, 'in_file')

            main_workflow.connect(
                rename_native_prob_wm, 'out_file',
                datasink, '@native_prob_wm')

            # rename prob_gm
            rename_native_prob_gm = pe.Node(
                niu.Rename(), name="rename_native_prob_gm")
            rename_native_prob_gm.inputs.format_string = \
                pref_deriv + "_space-native_label-GM_probseg"
            rename_native_prob_gm.inputs.parse_string = parse_str
            rename_native_prob_gm.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.native_prob_gm',
                rename_native_prob_gm, 'in_file')

            main_workflow.connect(
                rename_native_prob_gm, 'out_file',
                datasink, '@native_prob_gm')

            # rename prob_csf
            rename_native_prob_csf = pe.Node(
                niu.Rename(), name="rename_native_prob_csf")
            rename_native_prob_csf.inputs.format_string = \
                pref_deriv + "_space-native_label-CSF_probseg"
            rename_native_prob_csf.inputs.parse_string = parse_str
            rename_native_prob_csf.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.native_prob_csf',
                rename_native_prob_csf, 'in_file')

            main_workflow.connect(
                rename_native_prob_csf, 'out_file',
                datasink, '@native_prob_csf')

        if "brain_segment_pipe" in params:

            # rename segmented_brain_mask
            rename_native_segmented_brain_mask = pe.Node(
                niu.Rename(),
                name="rename_native_segmented_brain_mask")
            rename_native_segmented_brain_mask.inputs.format_string = \
                pref_deriv + "_space-native_desc-brain_dseg"
            rename_native_segmented_brain_mask.inputs.parse_string = parse_str
            rename_native_segmented_brain_mask.inputs.keep_ext = True

            main_workflow.connect(
                segment_pnh_pipe, 'outputnode.native_segmented_brain_mask',
                rename_native_segmented_brain_mask, 'in_file')

            main_workflow.connect(
                rename_native_segmented_brain_mask, 'out_file',
                datasink, '@native_segmented_brain_mask')

            # rename 5tt
            if "export_5tt_pipe" in params["brain_segment_pipe"].keys():
                rename_native_gen_5tt = pe.Node(
                    niu.Rename(), name="rename_native_gen_5tt")
                rename_native_gen_5tt.inputs.format_string = \
                    pref_deriv + "_space-native_desc-5tt_dseg"
                rename_native_gen_5tt.inputs.parse_string = parse_str
                rename_native_gen_5tt.inputs.keep_ext = True

                main_workflow.connect(
                    segment_pnh_pipe, 'outputnode.native_gen_5tt',
                    rename_native_gen_5tt, 'in_file')

                main_workflow.connect(
                    rename_native_gen_5tt, 'out_file',
                    datasink, '@native_gen_5tt')

            if "nii2mesh_brain_pipe" in params["brain_segment_pipe"] \
                    or "IsoSurface_brain_pipe" in params["brain_segment_pipe"]:

                print("Renaming wmgm_stl file")

                rename_native_wmgm_stl = pe.Node(
                    niu.Rename(), name="rename_native_wmgm_stl")
                rename_native_wmgm_stl.inputs.format_string = \
                    pref_deriv + "_desc-wmgm_mask"
                rename_native_wmgm_stl.inputs.parse_string = parse_str
                rename_native_wmgm_stl.inputs.keep_ext = True

                main_workflow.connect(
                    segment_pnh_pipe, 'outputnode.wmgm_stl',
                    rename_native_wmgm_stl, 'in_file')

                main_workflow.connect(
                    rename_native_wmgm_stl, 'out_file',
                    datasink, '@native_wmgm_stl')

                print("Renaming wmgm_nii file")
                rename_native_wmgm_mask = pe.Node(
                    niu.Rename(), name="rename_native_wmgm_mask")
                rename_native_wmgm_mask.inputs.format_string = \
                    pref_deriv + "_space-native_desc-wmgm_mask"
                rename_native_wmgm_mask.inputs.parse_string = parse_str
                rename_native_wmgm_mask.inputs.keep_ext = True

                main_workflow.connect(
                    segment_pnh_pipe, 'outputnode.native_wmgm_mask',
                    rename_native_wmgm_mask, 'in_file')

                main_workflow.connect(
                    rename_native_wmgm_mask, 'out_file',
                    datasink, '@native_wmgm_mask')

        elif "old_segment_pipe" in params.keys():

            if "mask_from_seg_pipe" in params.keys():

                # rename segmented_brain_mask
                rename_native_segmented_brain_mask = pe.Node(
                    niu.Rename(),
                    name="rename_native_segmented_brain_mask")
                rename_native_segmented_brain_mask.inputs.format_string = \
                    pref_deriv + "_space-native_desc-brain_dseg"
                rename_native_segmented_brain_mask.inputs.parse_string = \
                    parse_str
                rename_native_segmented_brain_mask.inputs.keep_ext = True

                main_workflow.connect(
                    segment_pnh_pipe, 'outputnode.native_segmented_brain_mask',
                    rename_native_segmented_brain_mask, 'in_file')

                main_workflow.connect(
                    rename_native_segmented_brain_mask, 'out_file',
                    datasink, '@native_segmented_brain_mask')

                print("Renaming wmgm_stl file")

                rename_native_wmgm_stl = pe.Node(
                    niu.Rename(),
                    name="rename_native_wmgm_stl")
                rename_native_wmgm_stl.inputs.format_string = \
                    pref_deriv + "_desc-wmgm_mask"
                rename_native_wmgm_stl.inputs.parse_string = parse_str
                rename_native_wmgm_stl.inputs.keep_ext = True

                main_workflow.connect(
                    segment_pnh_pipe, 'outputnode.wmgm_stl',
                    rename_native_wmgm_stl, 'in_file')

                main_workflow.connect(
                    rename_native_wmgm_stl, 'out_file',
                    datasink, '@native_wmgm_stl')

                # rename 5tt
                if "export_5tt_pipe" in params["old_segment_pipe"].keys():
                    rename_native_gen_5tt = pe.Node(
                        niu.Rename(), name="rename_native_gen_5tt")
                    rename_native_gen_5tt.inputs.format_string = \
                        pref_deriv + "_space-native_desc-5tt_dseg"
                    rename_native_gen_5tt.inputs.parse_string = parse_str
                    rename_native_gen_5tt.inputs.keep_ext = True

                    main_workflow.connect(
                        segment_pnh_pipe, 'outputnode.native_gen_5tt',
                        rename_native_gen_5tt, 'in_file')

                    main_workflow.connect(
                        rename_native_gen_5tt, 'out_file',
                        datasink, '@native_gen_5tt')





from nipype.interfaces.base import (CommandLine, CommandLineInputSpec,
                                    TraitedSpec)
from nipype.interfaces.base import traits, File


import os
from nipype.interfaces.fsl.base import (FSLCommand, FSLCommandInputSpec)


class T1xT2BETInputSpec(FSLCommandInputSpec):

    t1_file = File(
        exists=True,
        desc='Whole-head T1w image',
        mandatory=True, position=0, argstr="-t1 %s")

    t2_file = File(
        exists=True,
        desc='Whole-head T2w image',
        mandatory=True, position=1, argstr="-t2 %s")

    opt_os = traits.String(
        "_BET", usedefault=True,
        desc="Suffix for the brain masked images (default is _BET)",
        argstr="-os %s", mandatory=False)

    aT2 = traits.Bool(
        False, usedefault=True, argstr="-aT2",
        desc="Will coregrister T2w to T1w using flirt. Output will have the\
            suffix provided. Will only work for spatially close images.",
        mandatory=False)

    # as -> opt_as, as is already a part of python keywords...
    opt_as = traits.String(
        "-in-T1w", usedefault=True, argstr="-as %s",
        desc="Suffix for T2w to T1w registration \
            (\"-in-T1w\" if not specified)",
        mandatory=False)

    n = traits.Int(
        1, usedefault=True,
        desc='n = the number of iterations BET will be run to find center of \
            gravity (n=1 if option -n is absent)',
        argstr="-n %d", mandatory=False)

    m = traits.Bool(
        False, usedefault=True, argstr="-m",
        desc="Will output the BET mask at the format \
              output_prefixT1_mask.nii.gz)",
        mandatory=False)

    ms = traits.String(
        "_mask", usedefault=True,
        desc="Suffix for the mask (default is _mask)",
        argstr="-ms %s", mandatory=False)

    c = traits.Int(
        desc='Will crop the inputs & outputs after brain extraction. c is the \
            space between the brain and the limits of the crop box expressed \
            in percentage of the brain size (eg. if the brain size is 200\
            voxels in one dimension and c=10: the sides of the brain in this \
            dimension will be 20 voxels away from the borders of the resulting\
            crop box in this dimension).',
        argstr="-c %d", mandatory=False)

    cs = traits.String(
        "_cropped", usedefault=True,
        desc="Suffix for the cropped images (default is _cropped)",
        argstr="-cs %s", mandatory=False)

    f = traits.Float(
        0.5, usedefault=True,
        desc='-f options of BET: fractional intensity threshold (0->1); \
            default=0.5; smaller values give larger brain outline estimates',
        argstr="-f %f", mandatory=False)

    g = traits.Float(
        0.0, usedefault=True, desc='-g options of BET:\
            vertical gradient in fractional intensity threshold (-1->1);\
            default=0; positive values give larger brain outline at\
            bottom, smaller at top',
        argstr="-g %f", mandatory=False)

    cog = traits.List(
        desc='For difficult cases, you can directly provide a center of \
            gravity. Only one iteration will be performed.',
        argstr="-cog %d %d %d", mandatory=False)

    k = traits.Bool(
        False, usedefault=True, argstr="-k",
        desc="Will keep temporary files",
        mandatory=False)

    p = traits.String(
        desc="Prefix for running FSL functions\
              (can be a path or just a prefix)",
        argstr="-p %s", mandatory=False)


class T1xT2BETOutputSpec(TraitedSpec):
    t1_brain_file = File(
        exists=True, desc="extracted brain from T1")

    t2_brain_file = File(
        exists=True, desc="extracted brain from T2")

    t2_coreg_file = File(
        desc="coreg T2 from T1")

    mask_file = File(
        desc="extracted mask from T1")

    t1_cropped_file = File(
        desc="extracted cropped brain from T1")

    t2_cropped_file = File(
        desc="extracted cropped brain from T2")


class T1xT2BET(FSLCommand):
    """
    Description: Brain extraction using T1 and T2 images

    Inputs:

        Mandatory:

            t1_file
                Whole-head T1w image
            t2_file
                Whole-head T2w image

        Optional:

            opt_os
                Suffix for the brain masked images (default is _BET)
            aT2
                Will coregrister T2w to T1w using flirt. Output will have the
                suffix provided. Will only work for spatially close images.
            opt_as
                Suffix for T2w to T1w registration (\"-in-T1w\" if not
                specified)
            n
                the number of iterations BET will be run to find center of
                gravity (n=1 if option -n is absent)
            m
                Will output the BET mask at the format
                output_prefixT1_mask.nii.gz)
            ms
                Suffix for the mask (default is _mask)
            c
                Will crop the inputs & outputs after brain extraction. c is the
                space between the brain and the limits of the crop box
                expressed in percentage of the brain size (eg. if the brain
                size is 200 voxels in one dimension and c=10: the sides of the
                brain in this dimension will be 20 voxels away from the borders
                of the resulting crop box in this dimension)
            cs
                Suffix for the cropped images (default is _cropped)
            f
                -f options of BET: fractional intensity threshold (0->1); \
                default=0.5; smaller values give larger brain outline estimates
            g
                '-g options of BET: \
                vertical gradient in fractional intensity threshold (-1->1);\
                default=0; positive values give larger brain outline at\
                bottom, smaller at top
            cog
                For difficult cases, you can directly provide a center of \
                gravity. Only one iteration will be performed.
            k
                Will keep temporary files
            p
                Prefix for running FSL functions (can be a path or just a
                prefix)

    Outputs:

        t1_brain_file
            extracted brain from T1
        t2_brain_file
            extracted brain from T2
        t2_coreg_file
            coreg T2 from T1
        mask_file
            extracted mask from T1
        t1_cropped_file
            extracted cropped brain from T1
        t2_cropped_file
            extracted cropped brain from T2


    """
    input_spec = T1xT2BETInputSpec
    output_spec = T1xT2BETOutputSpec

    path_to_current_dir = os.path.dirname(os.path.abspath(__file__))
    package_directory = os.path.dirname(path_to_current_dir)

    _cmd = 'bash {}/bash/T1xT2BET.sh'.format(package_directory)

    def _list_outputs(self):

        import os
        from nipype.utils.filemanip import split_filename as split_f

        outputs = self._outputs().get()

        t1_path, t1_fname, ext = split_f(self.inputs.t1_file)
        t2_path, t2_fname, ext = split_f(self.inputs.t2_file)

        if self.inputs.c:
            # !!!!warning, in Regis bash, only .nii.gz are handled
            outputs["t1_cropped_file"] = os.path.abspath(
                t1_fname + self.inputs.cs + ".nii.gz")

            if self.inputs.aT2:
                outputs["t2_cropped_file"] = os.path.abspath(
                    t2_fname + self.inputs.opt_as + self.inputs.cs + ".nii.gz")
            else:
                outputs["t2_cropped_file"] = os.path.abspath(
                    t2_fname + self.inputs.cs + ".nii.gz")

            outputs["t1_brain_file"] = os.path.abspath(
                t1_fname + self.inputs.opt_os + self.inputs.cs + ".nii.gz")
            outputs["t2_brain_file"] = os.path.abspath(
                t2_fname + self.inputs.opt_os + self.inputs.cs + ".nii.gz")

            if self.inputs.m:
                # !!!!warning, in Regis bash, only .nii.gz are handled
                outputs["mask_file"] = os.path.abspath(
                    t1_fname + self.inputs.opt_os + self.inputs.ms +
                    self.inputs.cs + ".nii.gz")

        else:

            outputs["t1_brain_file"] = os.path.abspath(
                t1_fname + self.inputs.opt_os + ".nii.gz")
            outputs["t2_brain_file"] = os.path.abspath(
                t2_fname + self.inputs.opt_os + ".nii.gz")

            if self.inputs.m:
                # !!!!warning, in Regis bash, only .nii.gz are handled
                outputs["mask_file"] = os.path.abspath(
                    t1_fname + self.inputs.opt_os + self.inputs.ms + ".nii.gz")

        if self.inputs.aT2:
            outputs["t2_coreg_file"] = os.path.abspath(
                t2_fname + self.inputs.opt_as + ".nii.gz")

        return outputs


# AtlasBREX
class AtlasBREXInputSpec(CommandLineInputSpec):

    import os

    path_to_current_dir = os.path.dirname(os.path.abspath(__file__))
    package_directory = os.path.dirname(path_to_current_dir)

    script_atlas_BREX = File('{}/bash/atlasBREX.sh'.format(
            package_directory), usedefault=True,
        exists=True,
        desc='atlasBREX script',
        mandatory=False, position=0, argstr="%s")

    NMT_SS_file = File(
        exists=True,
        desc='Skullstriped version of the template',
        mandatory=True, position=1, argstr="-b %s")

    NMT_file = File(
        exists=True,
        desc='template img',
        mandatory=True, position=2, argstr="-nb %s")

    t1_restored_file = File(
        exists=True,
        desc='T1 image to map',
        mandatory=True, position=3, argstr="-h %s")

    f = traits.Float(
        0.5, usedefault=True, desc='f', position=4, argstr="-f %f",
        mandatory=True)

    wrp = traits.String(
        "10,10,10", usedefault=True, desc="wrp", position=5, argstr="-wrp %s",
        mandatory=True)

    reg = traits.Enum(
        1, 0, 2, 3, usedefault=True, desc="Method: 0 = FNIRT w/ bending, \
            1 (default) = FNIRT membrane-energy regularization, \
            2 = ANTs/SyN w/, \
            3 = w/o N4Bias",
        position=6, argstr="-reg %d", mandatory=True)

    msk = traits.String(
        "a,0,0", usedefault=True, desc="msk", position=7, argstr="-msk %s",
        mandatory=True)

    dil = traits.Int(desc="dil", argstr="-dil %d")

    nrm = traits.Enum(1, 2, desc="nrm", argstr="-nrm %d")

    vox = traits.Int(desc="vox",  argstr="-vox %d")

    args = traits.String(desc="args", position=-1, argstr=" %s",
                         mandatory=False)


class AtlasBREXOutputSpec(TraitedSpec):

    brain_file = File(
        exists=True,
        desc="extracted brain from atlas_brex")


class AtlasBREX(CommandLine):
    """
    Description: Atlas based BrainExtraction

    Inputs:

        Mandatory:

            script_atlas_BREX:
                File, 'atlasBREX script'

            NMT_SS_file:
                File, 'Skullstriped version of the template',

            NMT_file:
                File, 'template img'

            t1_restored_file:
                File, 'T1 image to map'

            f:
                Float, default = 0.5, f


            reg :

                Enum, default = 1,0,2,3, "Method: 0 = FNIRT w/ bending, \
                1 (default) = FNIRT membrane-energy regularization, \
                2 = ANTs/SyN w/, \
                3 = w/o N4Bias"

            w:
                String, default = "10,10,10", "w"

            msk:
                String, default = "a,0,0", "msk",

    Outputs:

        brain_file:
            File, "extracted brain from atlas_brex"

    """
    input_spec = AtlasBREXInputSpec
    output_spec = AtlasBREXOutputSpec

    _cmd = 'bash'

    def _format_arg(self, name, spec, value):

        import os
        import shutil

        cur_dir = os.getcwd()

        if name == 'script_atlas_BREX':
            shutil.copy(value, cur_dir)
            value = os.path.split(value)[1]

        if name == 'NMT_SS_file':
            shutil.copy(value, cur_dir)
            value = os.path.split(value)[1]

        if name == 'NMT_file':
            shutil.copy(value, cur_dir)
            value = os.path.split(value)[1]

        if name == 't1_restored_file':
            shutil.copy(value, cur_dir)
            value = os.path.split(value)[1]

        return super(AtlasBREX, self)._format_arg(name, spec, value)

    def _list_outputs(self):

        import os
        from nipype.utils.filemanip import split_filename as split_f

        outputs = self._outputs().get()

        path, fname, ext = split_f(self.inputs.t1_restored_file)
        outputs["brain_file"] = os.path.abspath(fname + '_brain' + ext)

        return outputs


if __name__ == '__main__':
    ab = AtlasBREX()

    print(ab.cmdline)





import nipype.pipeline.engine as pe

from nipype.interfaces.niftyreg.regutils import RegResample


def pad_back(seg_pipe, data_preparation_pipe, inputnode,
             node, nodefile,
             outputnode, outputnodefile,
             params, inter_val="NN"):

    pad_nodename = "pad_" + outputnodefile

    if "short_preparation_pipe" in params.keys():

        print("Using reg_aladin transfo to pad mask back")
        pad_node = pe.Node(
            RegResample(inter_val=inter_val),
            name=pad_nodename)

        seg_pipe.connect(
            node, nodefile,
            pad_node, "flo_file")

        seg_pipe.connect(
            data_preparation_pipe, "outputnode.native_T1",
            pad_node, "ref_file")

        seg_pipe.connect(
            data_preparation_pipe, "inv_tranfo.out_file",
            pad_node, "trans_file")

        # outputnode
        seg_pipe.connect(
            pad_node, "out_file",
            outputnode, outputnodefile)

    return pad_node


def apply_to_stereo(seg_pipe, native_to_stereo_pipe,
                    pad_node, pad_nodefile,
                    outputnode, outputnodefile):

    apply_nodename = "apply_" + outputnodefile

    # apply stereo to masked_debiased_T1
    apply_node = pe.Node(
        RegResample(pad_val=0.0),
        name=apply_nodename)

    seg_pipe.connect(
        pad_node, pad_nodefile,
        apply_node, "flo_file")

    seg_pipe.connect(
        native_to_stereo_pipe,
        'outputnode.native_to_stereo_trans',
        apply_node, "trans_file")

    seg_pipe.connect(
        native_to_stereo_pipe,
        'outputnode.stereo_native_T1',
        apply_node, "ref_file")

    # output
    seg_pipe.connect(
        apply_node, "out_file",
        outputnode, outputnodefile)

    return apply_node



"""
    Support function for loading test datasets
"""
import os
import os.path as op

import shutil
import json
import subprocess


def _download_data_zip(data_zip, name):

    json_data = op.join(op.dirname(op.abspath(__file__)),
                        "data_test_servers.json")

    data_dict = json.load(open(json_data))

    for key, cloud_elem in data_dict.items():
        print(key)

        data_dir = cloud_elem["data_dir"]

        if name not in data_dir.keys():
            print("{} not found in {}".format(name, key))
            continue
        server = cloud_elem["server"]

        if "curl_cloud_format" in list(cloud_elem.keys()):
            oc_path = cloud_elem["curl_cloud_format"].format(server,
                                                             data_dir[name])

            cmd = 'curl  {} --output {} '.format(oc_path, data_zip)

        else:
            if "cloud_format" in list(cloud_elem.keys()):
                oc_path = cloud_elem["cloud_format"].format(server,
                                                            data_dir[name])
            elif "cloud_format_3" in list(cloud_elem.keys()):
                oc_path = cloud_elem["cloud_format_3"].format(
                    server, data_dir[name], name)

            cmd = 'wget --no-check-certificate  \
                --content-disposition  {} -O {} '.format(oc_path, data_zip)

        val = subprocess.call(cmd.split())

        if val:
            print("Error with {} for {}".format(cmd, key))
            continue

        if op.exists(data_zip):
            print(os.listdir(op.split(data_zip)[0]))

            print("Ok for download {} with {}".format(data_zip, key))
            print("Quitting download function")

            return True

    assert op.exists(data_zip), \
        "Error, data_zip = {} not found ".format(data_zip)

    return False


def load_test_data(name, path_to=""):
    """ Load test data, template and needed scripts """

    if path_to == "":
        path_to = op.expanduser("~")

    assert op.exists(path_to), "Breaking, {} do not exist".format(path_to)

    data_dirpath = op.join(path_to, "data_macapype")

    try:
        os.makedirs(data_dirpath)
    except OSError:
        print("data_dirpath {} already exists".format(data_dirpath))

    data_path = op.join(data_dirpath, name)

    if op.exists(data_path):
        print("{} Already exists, skipping download".format(data_path))
        return data_path

    data_zip = op.join(data_dirpath, "{}.zip".format(name))

    if not op.exists(data_zip):

        print("Download {}".format(data_zip))

        val = _download_data_zip(data_zip, name)

        assert val, "Error, cannot download {}".format(data_zip)

    assert op.exists(data_zip), "Error, cannot find {}".format(data_zip)

    os.system("ls -l {}".format(data_dirpath))
    print("Unzip {} to {}".format(data_zip, data_path))
    os.system("unzip -o {} -d {}".format(data_zip, data_path))
    os.remove(data_zip)

    assert op.exists(data_path), "Error, cannot find {}".format(data_path)

    return data_path


def format_template(data_path, template_name):

    import json

    json_template = op.join(op.dirname(op.abspath(__file__)),
                            "templates.json")

    template_path_dict = json.load(open(json_template))

    assert template_name in template_path_dict.keys(), \
        "Error, could not find template formating for {} in {}".format(
            template_name, template_path_dict.keys())
    template_dict = template_path_dict[template_name]
    print("Found template formating for {}:".format(template_name))
    print(template_dict)

    for key, value in template_dict.items():
        template_file = op.join(data_path, value)
        assert op.exists(template_file), "Error, file {} is missing".format(
            template_file)

        template_dict[key] = template_file

    return template_dict


def make_tmp_dir():
    tmp_dir = "/tmp/test_macapype"
    if op.exists(tmp_dir):
        shutil.rmtree(tmp_dir)
    os.makedirs(tmp_dir)
    os.chdir(tmp_dir)

    return tmp_dir



#!/usr/bin/env python3
"""
    PNH anatomical segmentation pipeline given by Regis Trapeau warpped in
    Nipype.
    
    Description
    --------------
    TODO :/
    
    Arguments
    -----------
    -data:
        Path to the BIDS directory that contain subjects' MRI data.
        
    -resources:
        Path to the directory that contain resources (FSL/ and NMN_v1.2/).
        See 'Resources files' paragraph for further informations.
        
    -out:
        Nipype's processing directory. 
        It's where all the outputs will be saved.
        
    -subjects:
        IDs list of subjects to process.
    
    Example
    ---------
    python segment_pnh_regis.py -data [PATH_TO_BIDS] -out ../tests/ -subjects Elouk
    
    Resources files
    -----------------
    Brain templates are required to run this segmentation pipeline. Please,
    download the resources and specify the unzipped directory with the
    '-resources' argument in the command line.
    Resources are here:
    # TODO: find a permanent place
    https://cloud.int.univ-amu.fr/index.php/s/8bCJ5CWWPfHRyHs
    
    Requirements
    --------------
    This workflow use:
        - ANTS
        - SPM
        - FSL
    
"""

# Authors : David Meunier (david.meunier@univ-amu.fr)
#           Bastien Cagna (bastien.cagna@univ-amu.fr)
#           Regis Trapeau (regis.trapeau@univ-amu.fr)

import nipype.interfaces.io as nio
import nipype.interfaces.utility as niu
import nipype.pipeline.engine as pe

import nipype.interfaces.ants as ants
import nipype.interfaces.fsl as fsl

import os.path as op
import argparse

#import pybids
from bids.layout import BIDSLayout

from nipype.interfaces.io import BIDSDataGrabber

from macapype.nodes.denoise import nonlocal_denoise
from macapype.pipelines.register import create_iterative_register_pipe
from macapype.pipelines.extract_brain import create_old_segment_extraction_pipe
from macapype.utils.utils_tests import load_test_data

from macapype.utils.misc import get_first_elem

from macapype.utils.utils_spm import set_spm

assert set_spm(), "Error, SPM was not found, cannot run Regis pipeline"

fsl.FSLCommand.set_default_output_type('NIFTI_GZ')

def create_infosource(subject_ids):
    infosource = pe.Node(
        interface=niu.IdentityInterface(fields=['subject_id']),
        name="infosource"
    )
    infosource.iterables = [('subject_id', subject_ids)]

    return infosource


def create_datasource(data_dir, sess):
    datasource = pe.Node(
        interface=nio.DataGrabber(infields=['subject_id'], outfields=['T1']),
        name='datasource'
    )
    datasource.inputs.base_directory = data_dir
    #datasource.inputs.template = '%s/sub-%s_ses-01_%s.nii'
    datasource.inputs.template = 'sub-%s/{}/anat/sub-%s_{}_%sCropped.nii'.format(sess, sess)
    datasource.inputs.template_args = dict(
        T1=[['subject_id', 'subject_id', "T1w"]],
    )
    datasource.inputs.sort_filelist = True
    
    return datasource

def create_bids_datasource(data_dir):

    bids_datasource = pe.Node(
        interface=nio.BIDSDataGrabber(),
        name='bids_datasource'
    )

    bids_datasource.inputs.base_dir = data_dir

    bids_datasource.inputs.output_query = {'T1': {"datatype": "anat", "suffix": "T1w",
                        "extensions": ["nii", ".nii.gz"]}}

    layout = BIDSLayout(data_dir)
    print (layout)
    print(layout.get_subjects())
    print(layout.get_sessions())

    iterables = []
    if len(layout.get_subjects()) == 1:
         bids_datasource.inputs.subject = layout.get_subjects()[0]
    else:
        iterables.append(('subject', layout.get_subjects()[:2]))

    if len(layout.get_sessions()) == 1:
         bids_datasource.inputs.session = layout.get_sessions()[0]
    else:
        iterables.append(('session', layout.get_sessions()[:2]))

    if len(iterables):
        bids_datasource.iterables = iterables

    return bids_datasource

###############################################################################
def create_segment_pnh_onlyT1(nmt_file, nmt_ss_file, nmt_mask_file,
                              gm_prob_file, wm_prob_file, csf_prob_file,
                              name="segment_pnh_subpipes"):

    # creating pipeline
    seg_pipe = pe.Workflow(name=name)

    # creating inputnode
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['T1']),
        name='inputnode'
    )

    # N4 correction
    debias_N4 = pe.Node(ants.N4BiasFieldCorrection(), name='debias_N4')
    seg_pipe.connect(inputnode, 'T1', debias_N4, 'input_image')

    # N4 correction
    # iterative_debias_N4 = pe.Node(
    #     niu.Function(
    #         input_names=['input_image','stop_param'],
    #         output_names=['debiased_image'],
    #         function=interative_N4_debias),
    #     name='iterative_debias_N4')
    #
    # seg_pipe.connect(inputnode,'T1',iterative_debias_N4,'input_image')
    # iterative_debias_N4.inputs.stop_param = 0.01

    denoise_T1 = pe.Node(
      niu.Function(input_names=["img_file"],
                   output_names=["denoised_img_file"],
                   function=nonlocal_denoise),
      name="denoise_T1"
    )
    seg_pipe.connect(debias_N4, 'output_image', denoise_T1, 'img_file')

    # TODO: Attention , brain extraction should come in between !!!
    bet = pe.Node(fsl.BET(), name='bet')
    bet.inputs.frac = 0.7
    # TODO: specify center (depending of the subject)
    # bet.inputs.center = [,,]

    #seg_pipe.connect(debias_N4, 'output_image', bet, 'in_file') # without denoise
    seg_pipe.connect(denoise_T1,'denoised_img_file', bet, 'in_file')

    # Register template to anat (need also skullstripped anat)
    # use iterative flirt
    iterative_register_pipe = create_iterative_register_pipe(
        template_file=nmt_file,
        template_brain_file=nmt_ss_file,
        template_mask_file=nmt_mask_file,
        gm_prob_file=gm_prob_file,
        wm_prob_file=wm_prob_file,
        csf_prob_file=csf_prob_file,
        n_iter=1
    )

    seg_pipe.connect(
        bet, 'out_file',
        iterative_register_pipe, "inputnode.anat_file_BET"
    )
    seg_pipe.connect(
        debias_N4, 'output_image',
        iterative_register_pipe, 'inputnode.anat_file'
    )

    ## Compute brain mask using old_segment of SPM and postprocessing on
    ## tissues' masks
    extract_brain = create_old_segment_extraction_pipe()
    seg_pipe.connect(
        iterative_register_pipe, 'register.anat_file_brain',
        extract_brain, "inputnode.T1"
    )
    seg_pipe.connect(
        iterative_register_pipe, 'merge_3_files.list3files',
        extract_brain, "inputnode.seg_priors"
    )

    return seg_pipe


###############################################################################
def create_main_workflow(data_dir, process_dir, subject_ids, sess, nmt_dir, 
                         nmt_fsl_dir):
    """ """
    nmt_file = op.join(nmt_dir, "NMT.nii.gz")
    nmt_ss_file = op.join(nmt_dir, "NMT_SS.nii.gz")
    nmt_mask_file = op.join(nmt_dir, 'masks', 'anatomical_masks',
                            'NMT_brainmask.nii.gz')

    gm_prob_file = op.join(nmt_fsl_dir, 'NMT_SS_pve_1.nii.gz')
    wm_prob_file = op.join(nmt_fsl_dir, 'NMT_SS_pve_2.nii.gz')
    csf_prob_file = op.join(nmt_fsl_dir, 'NMT_SS_pve_3.nii.gz')

    main_workflow = pe.Workflow(name="test_pipeline_regis_denoise_bids")
    main_workflow.base_dir = process_dir

    # Infosource

    if subject_ids is None or sess is None:
        print('adding BIDS data source')

        datasource = create_bids_datasource(data_dir)

    else:

        print('adding info source and data source')
        infosource = create_infosource(subject_ids)

        # Data source
        datasource = create_datasource(data_dir, sess)

        # connect
        main_workflow.connect(infosource, 'subject_id', datasource, 'subject_id')


    # *************************** Preprocessing ********************************
    # segment_pnh
    print('adding segment_pnh')
    segment_pnh = create_segment_pnh_onlyT1(
        nmt_file, nmt_ss_file, nmt_mask_file, gm_prob_file, wm_prob_file,
        csf_prob_file
    )

    #main_workflow.connect(datasource, 'T1', segment_pnh, 'inputnode.T1')
    main_workflow.connect(datasource, ('T1', get_first_elem), segment_pnh, 'inputnode.T1')

    return main_workflow


################################################################################
def main(data_path, main_path, subjects, sess):
    data_path = op.abspath(data_path)
    #resources_dir = op.abspath(resources_dir)
    
    nmt_dir = load_test_data("NMT_v1.2")
    #nmt_dir = op.join(resources_dir, 'NMT_v1.2')
    
    # There also exists probabilistic maps in NMT_v1.2 folder,
    # do not know why Regis used these files he generated
    nmt_fsl_dir = load_test_data("NMT_FSL")
    
    # main_workflow
    print("Initialising the pipeline...")
    wf = create_main_workflow(
        data_dir=data_path,
        process_dir=main_path,
        subject_ids=subjects,
        sess=sess,
        nmt_dir=nmt_dir,
        nmt_fsl_dir=nmt_fsl_dir
    )
    #wf.write_graph(graph2use="colored")
    wf.config['execution'] = {'remove_unnecessary_outputs': 'false'}
    print('The PNH segmentation pipeline is ready')
    
    print("Start to process")
    wf.run()
    # wf.run(plugin='MultiProc', plugin_args={'n_procs' : 2})
    
if __name__ == '__main__':
    # Command line parser
    parser = argparse.ArgumentParser(
        description="PNH segmenation pipeline from Regis Trapeau")
    parser.add_argument("-data", dest="data", type=str, required=True,
                        help="Directory containing MRI data (BIDS)")
    parser.add_argument("-out", dest="out", type=str,
                        help="Output directory", required=True)
    parser.add_argument("-sess", dest="sess", type=str,
                        help="Session", required=False)
    parser.add_argument("-subjects", dest="subjects", type=str, nargs='+',
                        help="Subjects' ID", required=False)

    args = parser.parse_args()
    
    main(
        data_path=args.data,
        main_path=args.out,
        subjects=args.subjects,
        sess=args.sess
    )



"""
copied from https://stackoverrun.com/fr/q/6768689
seems was never wrapped in nipype
"""
import os
import json

from nipype.interfaces.fsl.base import FSLCommand, FSLCommandInputSpec

from nipype.interfaces.base import (CommandLine, CommandLineInputSpec,
                                    TraitedSpec, File, traits, Str)

from nipype.interfaces.afni.base import AFNICommandBase, AFNICommandOutputSpec


# Refit # rewritten from nipype to include -origin
# would be best to have a PR on nipype directly
class RefitInputSpec(CommandLineInputSpec):
    in_file = File(
        desc="input file to 3drefit",
        argstr="%s",
        position=-1,
        mandatory=True,
        exists=True,
        copyfile=True,
    )
    deoblique = traits.Bool(
        desc="replace current transformation matrix with cardinal matrix",
        argstr="-deoblique",
    )
    xorigin = Str(desc="x distance for edge voxeloffset", argstr="-xorigin %s")
    yorigin = Str(desc="y distance for edge voxeloffset", argstr="-yorigin %s")
    zorigin = Str(desc="z distance for edge voxeloffset", argstr="-zorigin %s")
    duporigin_file = File(
        argstr="-duporigin %s",
        exists=True,
        desc="Copies the xorigin, yorigin, and zorigin values from the header "
        "of the given dataset",
    )
    xdel = traits.Float(desc="new x voxel dimension in mm", argstr="-xdel %f")
    ydel = traits.Float(desc="new y voxel dimension in mm", argstr="-ydel %f")
    zdel = traits.Float(desc="new z voxel dimension in mm", argstr="-zdel %f")
    xyzscale = traits.Float(
        desc="Scale the size of the dataset voxels by the given factor",
        argstr="-xyzscale %f",
    )

    origin = traits.Str(
        argstr="-origin %s",
        desc="Defines the orientation e.g RSA, LPS, etc.")

    space = traits.Enum(
        "TLRC",
        "MNI",
        "ORIG",
        argstr="-space %s",
        desc="Associates the dataset with a specific template type, e.g. "
        "TLRC, MNI, ORIG",
    )
    atrcopy = traits.Tuple(
        File(exists=True),
        traits.Str(),
        argstr="-atrcopy %s %s",
        desc="Copy AFNI header attribute from the given file into the header "
        "of the dataset(s) being modified. For more information on AFNI "
        "header attributes, see documentation file README.attributes. "
        "More than one '-atrcopy' option can be used. For AFNI "
        "advanced users only. Do NOT use -atrcopy or -atrstring with "
        "other modification options. See also -copyaux.",
    )
    atrstring = traits.Tuple(
        traits.Str(),
        traits.Str(),
        argstr="-atrstring %s %s",
        desc="Copy the last given string into the dataset(s) being modified, "
        "giving it the attribute name given by the last string."
        "To be safe, the last string should be in quotes.",
    )
    atrfloat = traits.Tuple(
        traits.Str(),
        traits.Str(),
        argstr="-atrfloat %s %s",
        desc="Create or modify floating point attributes. "
        "The input values may be specified as a single string in quotes "
        "or as a 1D filename or string, example "
        "'1 0.2 0 0 -0.2 1 0 0 0 0 1 0' or "
        "flipZ.1D or '1D:1,0.2,2@0,-0.2,1,2@0,2@0,1,0'",
    )
    atrint = traits.Tuple(
        traits.Str(),
        traits.Str(),
        argstr="-atrint %s %s",
        desc="Create or modify integer attributes. "
        "The input values may be specified as a single string in quotes "
        "or as a 1D filename or string, example "
        "'1 0 0 0 0 1 0 0 0 0 1 0' or "
        "flipZ.1D or '1D:1,0,2@0,-0,1,2@0,2@0,1,0'",
    )
    saveatr = traits.Bool(
        argstr="-saveatr",
        desc="(default) Copy the attributes that are known to AFNI into "
        "the dset->dblk structure thereby forcing changes to known "
        "attributes to be present in the output. This option only makes "
        "sense with -atrcopy.",
    )
    nosaveatr = traits.Bool(argstr="-nosaveatr", desc="Opposite of -saveatr")


class Refit(AFNICommandBase):
    """Changes some of the information inside a 3D dataset's header

    For complete details, see the `3drefit Documentation.
    <https://afni.nimh.nih.gov/pub/dist/doc/program_help/3drefit.html>`_

    """

    _cmd = "3drefit"
    input_spec = RefitInputSpec
    output_spec = AFNICommandOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs["out_file"] = os.path.abspath(self.inputs.in_file)
        return outputs


# FslOrient
class FslOrientInputSpec(FSLCommandInputSpec):

    main_option = traits.Str(
        desc='main option', argstr='-%s', position=0, mandatory=True)

    code = traits.Int(
        argstr='%d', desc='code for setsformcode', position=1)

    in_file = File(
        exists=True, desc='input file', argstr='%s', position=2,
        mandatory=True)


class FslOrientOutputSpec(TraitedSpec):

    out_file = File(desc="out file", exists=True)


class FslOrient(FSLCommand):
    """
    copied and adapted from https://stackoverrun.com/fr/q/6768689
    seems was never wrapped in nipype
    """
    _cmd = 'fslorient'
    input_spec = FslOrientInputSpec
    output_spec = FslOrientOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs['out_file'] = os.path.abspath(self.inputs.in_file)
        return outputs


# CropVolume
class CropVolumeInputSpec(CommandLineInputSpec):

    # mandatory
    i_file = File(
        exists=True,
        desc='Volume to crop (you can specify as many -in as you want)',
        mandatory=True, position=0, argstr="-i %s")

    b_file = File(
        exists=True,
        desc='Brain image or brain mask, in the same space as the in-file(s)',
        mandatory=True, position=1, argstr="-b %s")

    # optional
    o = traits.String(
        desc="Prefix for the cropped image(s) (Must provide as many prefixes\
            as input images with -o, default is the base name of each input \
            image).",
        argstr="-o %s", mandatory=False)

    s = traits.String(
        "_cropped", usedefault=True,
        desc="Suffix for the cropped image(s) (default is \"_cropped\")",
        argstr="-s %s", mandatory=False)

    c = traits.Int(
        10, usedefault=True,
        desc='c is the space between the brain and the limits of\
            the crop box expressed in percentage of the brain size (eg. if the\
            brain size is 200 voxels in one dimension and c=10: the sides of\
            the brain in this dimension will be 20 voxels away from the\
            borders of the resulting crop box in this dimension). \
            Default: c=10',
        argstr="-c %d", mandatory=False)

    p = traits.String(
        desc="Prefix for running FSL functions\
            (can be a path or just a prefix)",
        argstr="-p %s", mandatory=False)


class CropVolumeOutputSpec(TraitedSpec):
    cropped_file = File(
        exists=True,
        desc="cropped image from CropVolume.sh")


class CropVolume(CommandLine):
    """
    Description: Crop image(s) based on a brain extraction. Multiple images
    can be cropped at once. Will crop each volume preceeded by the -i option

    Inputs:

        Mandatory:

            i_file
                Volume to crop (you can specify as many -in as you want)
            b_file
                Brain image or brain mask, in the same space as the in-file(s)

        Optional:

            o
                Prefix for the cropped image(s) (Must provide as many prefixes
                    as input images with -o, default is the base name of each
                    input image).
            s
                Suffix for the cropped image(s) (default is "_cropped")
            c
                c is the space between the brain and the limits of\
                    the crop box expressed in percentage of the brain size (eg.
                    if the brain size is 200 voxels in one dimension and c=10:
                    the sides of the brain in this dimension will be 20 voxels
                    away from the  borders of the resulting crop box in this
                    dimension). Default: c=10
            p
                Prefix for running FSL functions (can be a path or just a
                prefix)

    Outputs:

        cropped_file:

            cropped image from CropVolume.sh
    """
    input_spec = CropVolumeInputSpec
    output_spec = CropVolumeOutputSpec

    path_to_current_dir = os.path.dirname(os.path.abspath(__file__))
    package_directory = os.path.dirname(path_to_current_dir)

    _cmd = 'bash {}/bash/CropVolume.sh'.format(package_directory)

    def _list_outputs(self):

        import os
        from nipype.utils.filemanip import split_filename as split_f

        outputs = self._outputs().get()

        path, fname, ext = split_f(self.inputs.i_file)

        outfile = fname + self.inputs.s

        if self.inputs.o:
            outfile = self.inputs.o + outfile

        outputs["cropped_file"] = os.path.abspath(outfile + ".nii.gz")
        return outputs


###############################################################################
# Equivalent of flirt_average in FSL
def average_align(list_img, reorient=False):

    import os
    import nibabel as nib
    import numpy as np

    from nipype.utils.filemanip import split_filename as split_f
    import nipype.interfaces.fsl as fsl

    if isinstance(list_img, list):

        assert len(list_img) > 0, "Error, list should have at least one file"

        if len(list_img) == 1:
            assert os.path.exists(list_img[0])
            av_img_file = list_img[0]
        else:

            img_0 = nib.load(list_img[0])
            path, fname, ext = split_f(list_img[0])

            data_0 = img_0.get_fdata()

            data_0_shape = data_0.shape
            list_data = [data_0]

            for i, img in enumerate(list_img[1:]):
                data_orig_shape = nib.load(img).get_fdata().shape
                if data_orig_shape != data_0_shape:
                    print(f"**** Warning , original image shape \
                          {data_orig_shape} \
                          have a different shape than ref {data_0_shape}")

                print("running flirt on {}".format(img))
                flirt = fsl.FLIRT(dof=6)
                flirt.inputs.in_file = img
                flirt.inputs.reference = list_img[0]
                flirt.inputs.interp = "sinc"
                flirt.inputs.no_search = True
                out_file = flirt.run().outputs.out_file
                print(out_file)

                data = nib.load(out_file).get_fdata()
                assert data.shape == data_0_shape, \
                    f"**** Error, flirt results {out_file} \
                    have a different shape {data.shape} \
                    than ref {data_0_shape}"

                list_data.append(data)

            avg_data = np.mean(np.array(list_data), axis=0)

            assert data_0_shape == avg_data.shape, \
                f"Error with average shape {avg_data.shape} != {data_0_shape}"

            # after casting type
            avg_data = avg_data.astype(np.int16)

            av_img_file = os.path.abspath("avg_" + fname + ext)
            nib.save(nib.Nifti1Image(avg_data, header=img_0.header,
                                     affine=img_0.affine),
                     av_img_file)

    else:
        av_img_file = list_img

    assert os.path.exists(av_img_file)

    return av_img_file


# on the fly
def read_cropbox(cropbox_file):

    import os.path as op

    assert op.exists(cropbox_file), \
        "Error {} do not exists".format(cropbox_file)

    with open(cropbox_file) as f:
        crop_list = []
        for line in f.readlines():
            print(line.strip().split())
            crop_list.append(tuple(map(int, map(float, line.strip().split()))))

    return crop_list


def padding_cropped_img(cropped_img_file, orig_img_file, indiv_crop):

    import os

    import nibabel as nib
    import numpy as np

    from nipype.utils.filemanip import split_filename as split_f

    # cropping params
    crop = indiv_crop['crop_T1']['args'].split()
    print("cropping params {}".format(crop))

    xmin = int(crop[0])
    xmax = xmin + int(crop[1])

    ymin = int(crop[2])
    ymax = ymin + int(crop[3])

    zmin = int(crop[4])
    zmax = zmin + int(crop[5])

    # orig image
    orig_img = nib.load(orig_img_file)

    data_orig = orig_img.get_data()
    header_orig = orig_img.header
    affine_orig = orig_img.affine

    print("Orig img shape:", data_orig.shape)

    # cropped image
    cropped_img = nib.load(cropped_img_file)
    data_cropped = cropped_img.get_data()

    print("Cropped img shape:", data_cropped.shape)
    print("data_cropped.dtype: {}".format(data_cropped.dtype))

    if len(data_orig.shape) == 3 and len(data_cropped.shape) == 4:
        print("Padding with 3D params on a 4D image")

        padded_img_data_shape = (*data_orig.shape, data_cropped.shape[3])
        print(padded_img_data_shape)

        padded_img_data = np.zeros(shape=padded_img_data_shape,
                                   dtype=data_cropped.dtype)

        if not np.issubdtype(data_cropped.dtype, np.integer):
            print("Converting to NaN")
            padded_img_data[:] = np.nan

        print("Broscasted padded img shape:", padded_img_data_shape)

        for t in range(data_cropped.shape[3]):
            padded_img_data[xmin:xmax, ymin:ymax, zmin:zmax, t] = \
                data_cropped[:, :, :, t]

    else:
        print("Padding with 3D params on a 3D image")

        padded_img_data = np.zeros(shape=data_orig.shape,
                                   dtype=data_cropped.dtype)

        print("Padded img shape:", padded_img_data.shape)
        print("Padded_img_data.dtype: {}".format(padded_img_data.dtype))

        if not np.issubdtype(data_cropped.dtype, np.integer):
            print("Converting to NaN")
            padded_img_data[:] = np.nan

        padded_img_data[xmin:xmax, ymin:ymax, zmin:zmax] = data_cropped

        print("After Adding values")

    # saving padded image
    fpath, fname, ext = split_f(cropped_img_file)
    padded_img_file = os.path.abspath(fname + "_padded" + ext)
    img_padded_res = nib.Nifti1Image(padded_img_data, affine=affine_orig,
                                     header=header_orig)
    nib.save(img_padded_res, padded_img_file)

    return padded_img_file


def apply_li_thresh(orig_img_file):

    import os
    from skimage.filters import threshold_li
    import nibabel as nib
    import numpy as np

    from nipype.utils.filemanip import split_filename as split_f

    # orig image
    orig_img = nib.load(orig_img_file)

    data_orig = orig_img.get_fdata()
    header_orig = orig_img.header
    affine_orig = orig_img.affine

    print("Orig img shape:", data_orig.shape)

    print("Min/max values orig img :", np.min(data_orig), np.max(data_orig))

    lithresh = threshold_li(data_orig)

    img_lithresh_data = data_orig.astype(np.int16)

    img_lithresh_data[data_orig <= lithresh] = 0
    img_lithresh_data[lithresh < data_orig] = 1

    print("Min/max values after lithresh:", np.min(img_lithresh_data),
          np.max(img_lithresh_data))

    # saving lithresh image
    fpath, fname, ext = split_f(orig_img_file)
    lithr_img_file = os.path.abspath(fname + "_lithresh" + ext)
    img_lithresh = nib.Nifti1Image(img_lithresh_data,
                                   affine=affine_orig,
                                   header=header_orig)
    nib.save(img_lithresh, lithr_img_file)

    return lithr_img_file


if __name__ == '__main__':

    data_path = "/hpc/meca/data/Macaques/Macaque_hiphop/results/ucdavis"

    orig_file = os.path.join(data_path, "sub-032139/ses-001/anat/\
                             sub-032139_ses-001_run-1_T1w.nii.gz")

    mask_file = os.path.join(
        data_path,
        "derivatives/macapype_ANTS/sub-032139/ses-001/anat/\
        sub-032139_ses-001_space-orig_desc-brain_mask.nii.gz")

    indiv_file = os.path.join(
        "/hpc/meca/users/essamlali.a/Packages",
        "indiv_params_segment_macaque_ants_based_crop.json")
    indiv = json.load(open(indiv_file))

    indiv_crop = indiv['sub-032139']['ses-001']

    res = padding_cropped_img(mask_file, orig_file, indiv_crop)

    print(res)



from nipype.interfaces.base import (
    TraitedSpec, SimpleInterface, traits, File, CommandLineInputSpec)

from nipype.interfaces.afni.base import AFNICommandBase


def keep_gcc(nii_file):
    import os
    import nibabel as nib
    import numpy as np
    from nipype.utils.filemanip import split_filename as split_f

    def getLargestCC(segmentation):

        from skimage.measure import label

        labels = label(segmentation)
        assert labels.max() != 0  # assume at least 1 CC
        largestCC = labels == np.argmax(np.bincount(labels.flat)[1:])+1
        return largestCC, labels

    # nibabel (nifti -> np.array)
    img = nib.load(nii_file)
    data = img.get_fdata().astype(np.int16)

    # skimage GCC
    new_data, labels = getLargestCC(data)

    # nibabel (np.array -> nifti)

    path, fname, ext = split_f(nii_file)

    labels_img = nib.Nifti1Image(dataobj=labels,
                                 header=img.header,
                                 affine=img.affine)

    labels_file = os.path.abspath(fname + "_labels" + ext)

    nib.save(labels_img, labels_file)

    # nibabel (np.array -> nifti)
    new_img = nib.Nifti1Image(dataobj=new_data,
                              header=img.header,
                              affine=img.affine)

    gcc_nii_file = os.path.abspath(fname + "_gcc" + ext)

    nib.save(new_img, gcc_nii_file)

    return gcc_nii_file


def merge_tissues(dseg_file, keep_indexes):

    import os
    import nibabel as nib
    import numpy as np

    from nipype.utils.filemanip import split_filename as split_f

    print(dseg_file)

    dseg_img = nib.load(dseg_file)
    dseg_data = dseg_img.get_fdata()

    print(dseg_data.shape)
    print(dseg_data.dtype)
    print(np.unique(dseg_data))

    mask_data = np.zeros(shape=dseg_data.shape, dtype=np.int16)

    for index in keep_indexes:
        if index in np.unique(dseg_data):

            mask_data[dseg_data == index] = 1
        else:
            print("Error, could not find index {} in dseg file".format(index))

    print(np.unique(mask_data))

    mask_img = nib.Nifti1Image(dataobj=mask_data,
                               header=dseg_img.header,
                               affine=dseg_img.affine)

    path, fname, ext = split_f(dseg_file)

    mask_file = os.path.abspath(fname + "_mask" + ext)

    nib.save(mask_img, mask_file)

    return mask_file


# ### wrapping nii2mesh
def wrap_nii2mesh_old(nii_file):

    import os
    from nipype.utils.filemanip import split_filename as split_f

    path, fname, ext = split_f(nii_file)

    stl_file = os.path.abspath(fname + ".stl")

    cmd = "nii2mesh_old_gcc {} {}".format(nii_file, stl_file)

    ret = os.system(cmd)

    print(ret)

    assert ret == 0, "Error, cmd {} did not work".format(cmd)
    return stl_file


def wrap_nii2mesh(nii_file):

    import os
    from nipype.utils.filemanip import split_filename as split_f

    path, fname, ext = split_f(nii_file)

    stl_file = os.path.abspath(fname + ".stl")

    cmd = "nii2mesh {} {}".format(nii_file, stl_file)

    ret = os.system(cmd)

    print(ret)

    assert ret == 0, "Error, cmd {} did not work".format(cmd)
    return stl_file


# IsoSurface
class IsoSurfaceInputSpec(CommandLineInputSpec):

    nii_file = File(
        exists=True,
        desc='brain_file',
        mandatory=True, position=1, argstr="-input %s")

    stl_file = File(
        name_source=["nii_file"],
        name_template="%s.stl",
        desc="stl file",
        exists=True,
        keep_extension=False,
        position=2, argstr="-o %s")

    isoval = traits.Float(
        1.0,
        usedefault=True,
        desc='isoval',
        mandatory=True, argstr="-isoval %f")

    KPB = traits.Float(
        0.01,
        usedefault=True,
        desc='Smoothing ',
        requires=["NITER"],
        mandatory=True, position=-2, argstr="-Tsmooth %f")

    NITER = traits.Int(
        100,
        usedefault=True,
        desc='NITER',
        requires=["KPB"],
        mandatory=True, position=-1, argstr="%d")

    remesh = traits.Float(
        0.5, usedefault=True,
        desc='EDGE_FRACTION',
        mandatory=True, argstr="-remesh %f")

    autocrop = traits.Bool(
        True,
        desc='autocrop',
        mandatory=False, argstr="-autocrop")

    overwrite = traits.Bool(
        True,
        desc='overwrite',
        mandatory=False, argstr="-overwrite")


class IsoSurfaceOutputSpec(TraitedSpec):
    stl_file = File(
        exists=True, desc="stl file")


class IsoSurface(AFNICommandBase):
    """Description: Wrap of antsIsoSurface.sh

    Inputs:

        Mandatory:

            nii_file
                File, 'brain_file',

            stl_file:
                File, "stl file",

            isoval
                Float, 1, desc='isoval',

    ,       KPB
                Float, 0.01, 'Smoothing'

            NITER
                Int, 100 , 'NITER'

            remesh
                Float, 0.5 'EDGE_FRACTION',

        Optional:

            autocrop :
                Bool, 'autocrop',

            overwrite :
                Bool, 'overwrite'

    Outputs:

        stl_file :
            File, desc="stl file"

    """
    input_spec = IsoSurfaceInputSpec
    output_spec = IsoSurfaceOutputSpec

    _cmd = 'IsoSurface'


def split_LR_mask(LR_mask_file, left_index=1, right_index=2):

    import os

    import nibabel as nib
    import numpy as np

    from nipype.utils.filemanip import split_filename as split_f

    path, fname, ext = split_f(LR_mask_file)

    LR_mask_img = nib.load(LR_mask_file)

    LR_mask_data = LR_mask_img.get_fdata()

    # L_mask
    L_mask_data = np.zeros(shape=LR_mask_data.shape)

    L_mask_data[LR_mask_data == left_index] = 1

    L_mask_file = os.path.abspath("L_mask"+ext)

    nib.save(
        nib.Nifti1Image(L_mask_data, affine=LR_mask_img.affine,
                        header=LR_mask_img.header),
        L_mask_file)

    # R_mask
    R_mask_data = np.zeros(shape=LR_mask_data.shape)

    R_mask_data[LR_mask_data == right_index] = 1

    R_mask_file = os.path.abspath("R_mask"+ext)

    nib.save(
        nib.Nifti1Image(R_mask_data, affine=LR_mask_img.affine,
                        header=LR_mask_img.header),
        R_mask_file)

    return L_mask_file, R_mask_file


# Meshify
class MeshifyInputSpec(TraitedSpec):
    image_file = File(
        exists=True, desc='input file', mandatory=True)

    level = traits.Float(
        0.5, usedefault=True,
        desc='Threshold used to detect surfaces',
        mandatory=False)

    smoothing_iter = traits.Int(
        0, usedefault=True,
        desc='Number of Laplacian smoothing iterations',
        mandatory=False)

    smoothing_dt = traits.Float(
        0.1, usedefault=True,
        desc='dt param of the Laplacian smoothing',
        mandatory=False)


class MeshifyOutputSpec(TraitedSpec):
    mesh_file = File(desc="out file", exists=True)


class Meshify(SimpleInterface):
    """
    Definition: Create a Gifti mesh from a binary Nifti image
    """
    _cmd = 'fslorient'
    input_spec = MeshifyInputSpec
    output_spec = MeshifyOutputSpec

    def _run_interface(self, runtime):
        import os.path as op
        import nibabel.gifti as ng
        import numpy as np
        import skimage.measure as sm
        import nilearn.image as nimg

        import slam.io as sio
        import slam.differential_geometry as sdg

        from nipype.utils.filemanip import split_filename

        # Generate output mesh filename from the input image name
        _, fname, _ = split_filename(self.inputs.image_file)
        gii_file = op.abspath(op.join(runtime.cwd, fname + ".gii"))

        # Load the largest connected component of the input image
        img = nimg.largest_connected_component_img(self.inputs.image_file)

        # TODO: check if the input image is correct (binary)

        # Run the marching cube algorithm
        verts, faces, normals, values = sm.marching_cubes_lewiner(
            img.get_data(), self.inputs.level)

        # Convert vertices coordinates to image space
        # TODO: check that is correct by plotting the mesh on the image
        x, y, z = nimg.coord_transform(
            verts[:, 0], verts[:, 1], verts[:, 2], img.affine)
        mm_verts = np.array([x, y, z]).T

        # Save the mesh as Gifti
        # FIXME: FreeView can not open the mesh (but anatomist do)
        gii = ng.GiftiImage(darrays=[
            ng.GiftiDataArray(mm_verts, intent='NIFTI_INTENT_POINTSET'),
            ng.GiftiDataArray(faces, intent='NIFTI_INTENT_TRIANGLE')])
        gii.meta = ng.GiftiMetaData().from_dict({
            "volume_file": self.inputs.image_file,
            "marching_cube_level": str(self.inputs.level),
            "smoothing_iterations": str(self.inputs.smoothing_iter),
            "smoothing_dt": str(self.inputs.smoothing_dt)
        })
        ng.write(gii, gii_file)

        # Optional: Smooth the marching cube output with SLAM
        if self.inputs.smoothing_iter > 0:
            mesh = sdg.laplacian_mesh_smoothing(
                sio.load_mesh(gii_file),
                nb_iter=self.inputs.smoothing_iter,
                dt=self.inputs.smoothing_dt)
            sio.write_mesh(mesh, gii_file)

        return runtime




import os.path as op

from macapype.utils.utils_tests import make_tmp_dir
from macapype.utils.misc import parse_key
from macapype.pipelines.prepare import create_short_preparation_pipe

data_path = make_tmp_dir()


def test_create_crop_aladin_pipe_short_preparation_pipe():

    params = {
        "short_preparation_pipe":
        {
            "crop_aladin_pipe":
            {
            }
        }
    }

    params_template = {"template_head": ""}

    # running workflow
    segment_pnh = create_short_preparation_pipe(
        params_template=params_template,
        params=parse_key(params, "short_preparation_pipe"),
        name="short_manual_preparation_pipe")

    segment_pnh.base_dir = data_path

    segment_pnh.write_graph(graph2use="colored")
    assert op.exists(op.join(data_path,
                             "short_manual_preparation_pipe",
                             "graph.png"))


def test_create_crop_T1_short_preparation_pipe():

    params = {
        "short_preparation_pipe":
        {
            "crop_T1":
            {
                "args": ""
            }
        }
    }

    params_template = {"template_head": ""}
    # running workflow
    segment_pnh = create_short_preparation_pipe(
        params_template=params_template,
        params=parse_key(params, "short_preparation_pipe"),
        name="short_manual_preparation_pipe")

    segment_pnh.base_dir = data_path

    segment_pnh.write_graph(graph2use="colored")
    assert op.exists(op.join(data_path,
                             "short_manual_preparation_pipe",
                             "graph.png"))



pass

"""
import macapype.nodes.surface as surf
import numpy as np
import nibabel as nb
import os


def test_meshify():
    # Create a temporary test image
    print("Generate fake Nifti image")
    dt = np.zeros((100, 60, 100))
    dt[40:60, 20:40, 20:50] = 1
    affine = np.eye(4)

    tmp_file = "./test_meshify_input_image.nii"
    nb.save(nb.Nifti1Image(dt, affine), tmp_file)

    try:
        # Test the node
        print("Create Meshify node")
        meshify = surf.Meshify()
        meshify.inputs.image_file = tmp_file
        print("Run Meshify node")
        meshify.run()

        print("Assert output")
        assert os.path.exists(tmp_file[:-4] + ".gii")
        pass
    finally:
        print("Remove temporary data")
        os.remove(tmp_file)
        os.remove(tmp_file[:-4] + ".gii")

"""



import os
from nipype.interfaces.base import (CommandLine, CommandLineInputSpec,
                                    TraitedSpec)
from nipype.interfaces.base import traits, File


# T1xT2BiasFieldCorrection
class T1xT2BiasFieldCorrectionInputSpec(CommandLineInputSpec):

    t1_file = File(
        exists=True,
        desc='Whole-head T1w image',
        mandatory=True, position=0, argstr="-t1 %s")

    t2_file = File(
        exists=True,
        desc='Whole-head T2w image (use -aT2 if T2w image is not in the T1w \
              space)',
        mandatory=True, position=1, argstr="-t2 %s")

    os = traits.String(
        "_debiased", usedefault=True,
        desc="Suffix for the bias field corrected images (default is \
              \"_debiased\")",
        argstr="-os %s", mandatory=False)

    aT2 = traits.Bool(
        False, usedefault=True,
        argstr="-aT2",
        desc="Will coregrister T2w to T1w using flirt. Output will have the\
            suffix provided. Will only work for spatially close images.",
        mandatory=False)

    # as -> opt_as, as is already a part of python keywords...
    opt_as = traits.String(
        "-in-T1w", usedefault=True, argstr="-as %s",
        desc="Suffix for T2w to T1w registration \
              (\"-in-T1w\" if not specified)",
        mandatory=False)

    s = traits.Int(
        4, usedefault=True,
        desc='size of gauss kernel in mm when performing mean filtering \
              (default=4)',
        argstr="-s %d", mandatory=False)

    # exists = True ???
    b = traits.File(
        argstr="-b %s",
        desc="Brain mask file. Will also output bias corrected brain files \
            with the format \"output_prefix_brain.nii.gz\"",
        mandatory=False)

    bet = traits.Int(
        0, usedefault=True,
        desc='Will try to "smart" BET the anat files to get a brain mask: n =\
            the number of iterations BET will be run to find center of gravity\
            (default=0, will not BET if option -b has been specified).',
        argstr="-bet %d", mandatory=False)

    bs = traits.String(
        "_BET", usedefault=True,
        desc="Suffix for the BET masked images (default is \"_BET\")",
        argstr="-bs %s", mandatory=False)

    f = traits.Float(
        0.5, usedefault=True,
        desc='-f options of BET: fractional intensity threshold (0->1); \
            default=0.5; smaller values give larger brain outline estimates',
        argstr="-f %f", mandatory=False)

    g = traits.Float(
        0.0, usedefault=True, desc='-g options of BET:\
                  vertical gradient in fractional intensity threshold (-1->1);\
                  default=0; positive values give larger brain outline at\
                  bottom, smaller at top',
        argstr="-g %f", mandatory=False)

    k = traits.Bool(
        False, usedefault=True, argstr="-k",
        desc="Will keep temporary files",
        mandatory=False)

    p = traits.String(
        desc="Prefix for running FSL functions\
              (can be a path or just a prefix)",
        argstr="-p %s", mandatory=False)


class T1xT2BiasFieldCorrectionOutputSpec(TraitedSpec):
    t1_debiased_file = File(
        exists=True,
        desc="debiased T1")

    t2_debiased_file = File(
        exists=True,
        desc="debiased T2")

    t2_coreg_file = File(
        desc="T2 on T1")

    t1_debiased_brain_file = File(
        desc="debiased bet T1")

    t2_debiased_brain_file = File(
        desc="debiased bet T2")

    debiased_mask_file = File(
        desc="debiased bet mask")


class T1xT2BiasFieldCorrection(CommandLine):
    """
    Description:  Bias field correction using T1w & T2w images.
        Provides an attempt of brain extration if wanted.

    Inputs:

        Mandatory:

            t1_file
                Whole-head T1w image
            t2_file
                Whole-head T2w image (use -aT2 if T2w image is not in the T1w
                space)

        Optional:

            os
                Suffix for the bias field corrected images (default is
                "_debiased")
            aT2
                Will coregrister T2w to T1w using flirt. Output will have the\
                    suffix provided. Will only work for spatially close images.
            opt_as
                Suffix for T2w to T1w registration ("-in-T1w" if not specified)
            s
                size of gauss kernel in mm when performing mean filtering \
                    (default=4)argstr="-s %d", mandatory=False)
            b
                Brain mask file. Will also output bias corrected brain files \
                    with the format "output_prefix_brain.nii.gz"
            bet
                Will try to "smart" BET the anat files to get a brain mask:
                n = the number of iterations BET will be run to find center of
                gravity (default=0, will not BET if option -b has been
                specified).
            bs
                Suffix for the BET masked images (default is "_BET")
            f
                -f options of BET: fractional intensity threshold (0->1); \
                    default=0.5; smaller values give larger brain outline
                    estimates
            g
                -g options of BET:\
                        vertical gradient in fractional intensity threshold
                        (-1->1); default=0; positive values give larger brain
                        outline at bottom, smaller at top
            k
                Will keep temporary files
            p
                Prefix for running FSL functions\
                    (can be a path or just a prefix)

    Outputs:

        t1_debiased_file
            debiased T1
        t2_debiased_file
            debiased T2
        t2_coreg_file
            T2 on T1
        t1_debiased_brain_file
            debiased bet T1
        t2_debiased_brain_file
            debiased bet T2
        debiased_mask_file
            debiased bet mask
    """
    input_spec = T1xT2BiasFieldCorrectionInputSpec
    output_spec = T1xT2BiasFieldCorrectionOutputSpec

    package_directory = os.path.dirname(os.path.abspath(__file__))
    _cmd = 'bash {}/../bash/T1xT2BiasFieldCorrection.sh'.format(
        package_directory)

    def _list_outputs(self):

        import os
        from nipype.utils.filemanip import split_filename as split_f

        outputs = self._outputs().get()

        t1_path, t1_fname, ext = split_f(self.inputs.t1_file)
        t2_path, t2_fname, ext = split_f(self.inputs.t2_file)

        # !!!!warning, in Regis bash, only .nii.gz are handled
        outputs["t1_debiased_file"] = os.path.abspath(
            t1_fname + self.inputs.os + ".nii.gz")
        outputs["t2_debiased_file"] = os.path.abspath(
            t2_fname + self.inputs.os + ".nii.gz")

        if self.inputs.aT2:
            outputs["t2_coreg_file"] = os.path.abspath(
                t2_fname + self.inputs.opt_as + ".nii.gz")

        if self.inputs.bet:
            outputs["t1_debiased_brain_file"] = os.path.abspath(
                t1_fname + self.inputs.os + self.inputs.bs + ".nii.gz")
            outputs["t2_debiased_brain_file"] = os.path.abspath(
                t2_fname + self.inputs.os + self.inputs.bs + ".nii.gz")
            outputs["debiased_mask_file"] = os.path.abspath(
                t1_fname + self.inputs.os + self.inputs.bs + "_mask.nii.gz")
        elif self.inputs.b:
            outputs["t1_debiased_brain_file"] = os.path.abspath(
                t1_fname + self.inputs.os + "_brain.nii.gz")
            outputs["t2_debiased_brain_file"] = os.path.abspath(
                t2_fname + self.inputs.os + "_brain.nii.gz")
        return outputs

# ############## simpleITK debias


def itk_debias(img_file):

    import os
    import SimpleITK as sitk

    from nipype.utils.filemanip import split_filename as split_f

    # Read the input image
    input_image = sitk.ReadImage(img_file, sitk.sitkFloat32)

    # Initialize the N4 bias field correction filter
    n4_filter = sitk.N4BiasFieldCorrectionImageFilter()

    # best paramters for marmoset Randy petra
    n4_filter.SetMaximumNumberOfIterations([200, 100, 50, 25])
    n4_filter.SetConvergenceThreshold(0.0001)
    n4_filter.SetWienerFilterNoise(0.01)

    input_image_mask = sitk.LiThreshold(input_image, 0, 1)

    # the output image
    corrected_image = n4_filter.Execute(input_image, input_image_mask)

    # get the bias field
    bias_field_image = n4_filter.GetLogBiasFieldAsImage(input_image)

    # Save the corrected image and bias
    fpath, fname, ext = split_f(img_file)
    cor_img_file = os.path.abspath(fname + "_debias" + ext)

    bias_img_file = os.path.abspath(fname + "_bias" + ext)

    sitk.WriteImage(corrected_image, cor_img_file)

    sitk.WriteImage(bias_field_image, bias_img_file)

    return cor_img_file, bias_img_file



"""
    Pipelines for brain extraction

"""
import nipype.interfaces.utility as niu
import nipype.pipeline.engine as pe

import nipype.interfaces.fsl as fsl

from ..nodes.extract_brain import AtlasBREX

from ..utils.utils_nodes import NodeParams, parse_key


def create_extract_pipe(params_template, params={},
                        name="extract_T1_pipe"):
    """
    Description: Extract T1 brain using AtlasBrex

    Params:

    - norm_intensity (see `N4BiasFieldCorrection <https://nipype.readthedocs.\
    io/en/0.12.1/interfaces/generated/nipype.interfaces.ants.segmentation.html\
    #n4biasfieldcorrection>`_ for arguments)
    - atlas_brex (see :class:`AtlasBREX \
    <macapype.nodes.extract_brain.AtlasBREX>` for arguments) - also available \
    as :ref:`indiv_params <indiv_params>`

    Inputs:

        inputnode:

            restore_T1:
                preprocessed (debiased/denoised) T1 file name

        arguments:

            params_template:
                dictionary of info about template

            params:
                dictionary of node sub-parameters (from a json file)

            name:
                pipeline name (default = "extract_pipe")

    Outputs:

        smooth_mask.out_file:
            Computed mask (after some smoothing)

    """

    # creating pipeline
    extract_pipe = pe.Workflow(name=name)

    # creating inputnode
    inputnode = pe.Node(
        niu.IdentityInterface(fields=['restore_T1',
                                      "indiv_params"]),
        name='inputnode')

    # smooth before brex
    if "smooth" in params.keys():

        smooth = NodeParams(fsl.utils.Smooth(),
                            params=parse_key(params, "smooth"),
                            name='smooth')

        extract_pipe.connect(inputnode, 'restore_T1',
                             smooth, 'in_file')
    # atlas_brex
    atlas_brex = NodeParams(AtlasBREX(),
                            params=parse_key(params, "atlas_brex"),
                            name='atlas_brex')

    if "smooth" in params.keys():
        extract_pipe.connect(smooth, 'smoothed_file',
                             atlas_brex, 't1_restored_file')
    else:

        extract_pipe.connect(inputnode, 'restore_T1',
                             atlas_brex, 't1_restored_file')

    atlas_brex.inputs.NMT_file = params_template["template_head"]
    atlas_brex.inputs.NMT_SS_file = params_template["template_brain"]

    extract_pipe.connect(
            inputnode, ("indiv_params", parse_key, "atlas_brex"),
            atlas_brex, 'indiv_params')

    # mask_brex
    mask_brex = pe.Node(fsl.UnaryMaths(), name='mask_brex')
    mask_brex.inputs.operation = 'bin'

    extract_pipe.connect(atlas_brex, 'brain_file', mask_brex, 'in_file')

    # smooth_mask
    smooth_mask = pe.Node(fsl.UnaryMaths(), name='smooth_mask')
    smooth_mask.inputs.operation = "bin"
    smooth_mask.inputs.args = "-s 1 -thr 0.5 -bin"

    extract_pipe.connect(mask_brex, 'out_file', smooth_mask, 'in_file')

    return extract_pipe



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Pydra: Dataflow Engine

"""
import sys
from setuptools import setup

import re

def _get_version():

    verstr = "unknown"
    try:
        verstrline = open('macapype/_version.py', "rt").read()
    except EnvironmentError:
        pass  # Okay, there is no version file.
    else:
        VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
        mo = re.search(VSRE, verstrline, re.M)
        if mo:
            verstr = mo.group(1)
        else:
            raise RuntimeError("unable to find version in yourpackage/_version.py")
    return verstr

if __name__ == "__main__":
    setup(
        version=_get_version()
    )



import os
import shutil

import os.path as op

import pytest

from macapype.nodes.extract_brain import T1xT2BET

from macapype.utils.utils_tests import (load_test_data, format_template,
                                        make_tmp_dir)
from macapype.utils.utils_nodes import NodeParams


def test_server_amubox():
    tmp_path = make_tmp_dir()
    name = op.join(tmp_path, "data_test_macaque.zip")

    code = "BdpeSWACJbrNp8r"
    server = "https://amubox.univ-amu.fr"
    add = "{}/public.php?service=files&t={}&download".format(server, code)
    cmd = "wget  --no-check-certificate  \"{}\" -O {}".format(add, name)

    os.system(cmd)


def test_load_test_data():
    with pytest.raises(AssertionError):
        load_test_data("do_not_exists")


def test_load_test_data_dataset():

    template_name = "NMT_v2.0_asym"
    nmt_dir = load_test_data(name=template_name)
    params_template = format_template(nmt_dir, template_name)

    assert len(params_template) != 0


def test_data_test_macaque():

    data_path = load_test_data("data_test_macaque")

    T1_file = op.join(data_path, "sub-Apache_ses-01_T1w.nii")
    T2_file = op.join(data_path, "sub-Apache_ses-01_T2w.nii")

    params = {"t1_file": T1_file, "t2_file": T2_file, "aT2": True}

    bet_crop = NodeParams(interface=T1xT2BET(), params=params,  # noqa
                          name="bet_crop")


def test_zenodo_server():

    test_file = "Juna_Chimp_T1_1mm_skull.nii.gz"
    download_file = os.path.abspath(test_file)

    os.system("wget --no-check-certificate  --content-disposition \
        https://zenodo.org/record/4683381/files/{}?download=1 \
        -O {}".format(test_file, download_file))

    assert os.path.exists(download_file)

    os.remove(download_file)


def test_load_test_data_zenodo():

    tmp_path = make_tmp_dir()
    name = "Juna_Chimp"
    data_path = load_test_data(name=name, path_to=tmp_path)

    print(data_path)

    assert os.path.exists(data_path)
    assert len(os.listdir(data_path))

    shutil.rmtree(data_path)


if __name__ == '__main__':

    # test_load_test_data()
    # test_data_test_macaque()

    test_zenodo_server()
    test_load_test_data_zenodo()
